@@ -1,4 +1,21 @@
-/* mbox.c */
+/*
+ * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
+ * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -17,10 +34,12 @@
 #include "intl.h"
 #include "mbox.h"
 #include "procmsg.h"
+#include "folder.h"
+#include "filter.h"
+#include "prefs_common.h"
 #include "utils.h"
 
 #define MSGBUFSIZE	8192
-#define TMPMSG 		".tmpmsg"
 
 #define FPUTS_TO_TMP_ABORT_IF_FAIL(s) \
 { \
@@ -28,20 +47,25 @@
 		g_warning(_("can't write to temporary file\n")); \
 		fclose(tmp_fp); \
 		fclose(mbox_fp); \
+		unlink(tmp_file); \
 		return -1; \
 	} \
 }
 
-gint proc_mbox(const gchar *dest, const gchar *mbox)
+gint proc_mbox(FolderItem *dest, const gchar *mbox, GHashTable *folder_table)
 {
 	FILE *mbox_fp;
 	gchar buf[MSGBUFSIZE], from_line[MSGBUFSIZE];
-	gint msgnum, msgs = 0;
+	gchar *tmp_file;
+	gint msgs = 0;
 
-	debug_print(_("Getting messages from %s into %s...\n"), mbox, dest);
+	g_return_val_if_fail(dest != NULL, -1);
+	g_return_val_if_fail(mbox != NULL, -1);
+
+	debug_print(_("Getting messages from %s into %s...\n"), mbox, dest->path);
 
 	if ((mbox_fp = fopen(mbox, "r")) == NULL) {
-		perror("fopen");
+		FILE_OP_ERROR(mbox, "fopen");
 		return -1;
 	}
 
@@ -67,32 +91,25 @@ gint proc_mbox(const gchar *dest, const gchar *mbox)
 		return -1;
 	}
 
-	if ((msgnum = procmsg_get_last_message_number(dest)) < 0) {
-		g_warning(_("can't get last message number in %s\n"), dest);
-		fclose(mbox_fp);
-		return -1;
-	}
-	msgnum++;
+	tmp_file = get_tmp_file();
 
 	do {
 		FILE *tmp_fp;
-		gchar *path;
+		FolderItem *dropfolder;
 		gchar *startp, *endp, *rpath;
 		gint empty_line;
+		gint val;
 
-		if ((tmp_fp = fopen(TMPMSG, "w")) == NULL) {
-			perror("fopen");
+		if ((tmp_fp = fopen(tmp_file, "w")) == NULL) {
+			FILE_OP_ERROR(tmp_file, "fopen");
 			g_warning(_("can't open temporary file\n"));
 			fclose(mbox_fp);
 			return -1;
 		}
-		if (chmod(TMPMSG, S_IRUSR|S_IWUSR) < 0) {
-			perror("chmod");
-			g_warning(_("can't change file mode\n"));
-		}
+		if (chmod(tmp_file, S_IRUSR|S_IWUSR) < 0)
+			FILE_OP_ERROR(tmp_file, "chmod");
 
 		/* convert unix From into Return-Path */
-		//g_print("%s/%d:\n%s%s\n", dest, msgnum, from_line, buf);
 		startp = from_line + 5;
 		endp = strchr(startp, ' ');
 		if (endp == NULL)
@@ -111,18 +128,19 @@ gint proc_mbox(const gchar *dest, const gchar *mbox)
 		empty_line = 0;
 
 		while (fgets(buf, sizeof(buf), mbox_fp) != NULL) {
-			if (buf[0] == '\n') {
+			if (buf[0] == '\n' || buf[0] == '\r') {
 				empty_line++;
 				buf[0] = '\0';
 				continue;
 			}
 
 			/* From separator */
-			if (empty_line > 0 && !strncmp(buf, "From ", 5)) {
+			if (!strncmp(buf, "From ", 5)) {
 				strcpy(from_line, buf);
 				if (fgets(buf, sizeof(buf), mbox_fp) != NULL) {
 					if (is_header_line(buf)) break;
-					g_warning(_("unescaped From found:\n%s"), from_line);
+					else
+						g_warning(_("unescaped From found:\n%s"), from_line);
 				} else
 					buf[0] = '\0';
 			}
@@ -153,25 +171,42 @@ gint proc_mbox(const gchar *dest, const gchar *mbox)
 				FPUTS_TO_TMP_ABORT_IF_FAIL("\n");
 		}
 
-		fclose(tmp_fp);
-
-		/* move the temporary file to the destination */
-		path = g_strdup_printf("%s%c%d",
-				       dest, G_DIR_SEPARATOR, msgnum);
-		if (rename(TMPMSG, path) < 0) {
-			perror("rename");
-			g_warning(_("can't move tmpmsg to %s\n"), path);
-			unlink(TMPMSG);
+		if (fclose(tmp_fp) == EOF) {
+			FILE_OP_ERROR(tmp_file, "fclose");
+			g_warning(_("can't write to temporary file\n"));
 			fclose(mbox_fp);
-			g_free(path);
+			unlink(tmp_file);
 			return -1;
 		}
-		g_free(path);
-		msgnum++;
+
+		if (folder_table) {
+			dropfolder = filter_get_dest_folder
+				(prefs_common.fltlist, tmp_file);
+			if (!dropfolder ||
+			    !strcmp(dropfolder->path, FILTER_NOT_RECEIVE))
+				dropfolder = dest;
+			val = GPOINTER_TO_INT(g_hash_table_lookup
+					      (folder_table, dropfolder));
+			if (val == 0) {
+				folder_item_scan(dropfolder);
+				g_hash_table_insert(folder_table, dropfolder,
+						    GINT_TO_POINTER(1));
+			}
+		} else
+			dropfolder = dest;
+
+		if (folder_item_add_msg(dropfolder, tmp_file) < 0) {
+			fclose(mbox_fp);
+			unlink(tmp_file);
+			return -1;
+		}
+
 		msgs++;
 	} while (from_line[0] != '\0');
 
+	fclose(mbox_fp);
 	debug_print(_("%d messages found.\n"), msgs);
+
 	return msgs;
 }
 
@@ -186,7 +221,7 @@ gint lock_mbox(const gchar *base, LockType type)
 
 		lockfile = g_strdup_printf("%s.%d", base, getpid());
 		if ((lockfp = fopen(lockfile, "w")) == NULL) {
-			perror("fopen");
+			FILE_OP_ERROR(lockfile, "fopen");
 			g_warning(_("can't create lock file %s\n"), lockfile);
 			g_warning(_("use 'flock' instead of 'file' if possible.\n"));
 			g_free(lockfile);
@@ -198,8 +233,8 @@ gint lock_mbox(const gchar *base, LockType type)
 
 		locklink = g_strconcat(base, ".lock", NULL);
 		while (link(lockfile, locklink) < 0) {
-			perror("link");
-			if (retry >= 10) {
+			FILE_OP_ERROR(lockfile, "link");
+			if (retry >= 5) {
 				g_warning(_("can't create %s\n"), lockfile);
 				unlink(lockfile);
 				g_free(lockfile);
@@ -221,8 +256,7 @@ gint lock_mbox(const gchar *base, LockType type)
 #else
 		if ((lockfd = open(base, O_RDWR)) < 0) {
 #endif
-			perror("open");
-			g_warning(_("can't open %s\n"), base);
+			FILE_OP_ERROR(base, "open");
 			return -1;
 		}
 #if HAVE_FLOCK
@@ -257,8 +291,7 @@ gint unlock_mbox(const gchar *base, gint fd, LockType type)
 
 		lockfile = g_strconcat(base, ".lock", NULL);
 		if (unlink(lockfile) < 0) {
-			perror("unlink");
-			g_warning(_("can't unlink lock file %s\n"), lockfile);
+			FILE_OP_ERROR(lockfile, "unlink");
 			g_free(lockfile);
 			return -1;
 		}
@@ -305,9 +338,9 @@ void empty_mbox(const gchar *mbox)
 	if (truncate(mbox, 0) < 0) {
 		FILE *fp;
 
-		perror("truncate");
+		FILE_OP_ERROR(mbox, "truncate");
 		if ((fp = fopen(mbox, "w")) == NULL) {
-			perror("fopen");
+			FILE_OP_ERROR(mbox, "fopen");
 			g_warning(_("can't truncate mailbox to zero.\n"));
 			return;
 		}
