@@ -1,4 +1,21 @@
-/* prefs.c */
+/*
+ * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
+ * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -11,6 +28,7 @@
 #include <unistd.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <errno.h>
 
 #include "intl.h"
 #include "main.h"
@@ -41,11 +59,9 @@ void prefs_read_config(PrefParam *param, const gchar *label,
 
 	prefs_set_default(param);
 
-	rcpath = g_strconcat(g_get_home_dir(), G_DIR_SEPARATOR_S, RC_DIR,
-			     G_DIR_SEPARATOR_S, rcfile, NULL);
+	rcpath = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, rcfile, NULL);
 	if ((fp = fopen(rcpath, "r")) == NULL) {
-		fprintf(stderr, "%s: ", rcpath);
-		perror("fopen");
+		if (ENOENT != errno) FILE_OP_ERROR(rcpath, "fopen");
 		g_free(rcpath);
 		return;
 	}
@@ -116,42 +132,43 @@ void prefs_config_parse_one_line(PrefParam *param, const gchar *buf)
 	}
 }
 
+#define TRY(func) \
+if (!(func)) \
+{ \
+	g_warning(_("failed to write configuration to file\n")); \
+	if (orig_fp) fclose(orig_fp); \
+	prefs_write_close_revert(pfile); \
+	g_free(rcpath); \
+	g_free(block_label); \
+	return; \
+} \
+
 void prefs_save_config(PrefParam *param, const gchar *label,
 		       const gchar *rcfile)
 {
-	FILE *orig_fp, *tmp_fp;
-	gchar *rcpath, *tmprcpath;
+	FILE *orig_fp;
+	PrefFile *pfile;
+	gchar *rcpath;
 	gchar buf[PREFSBUFSIZE];
-	gchar *block_label;
+	gchar *block_label = NULL;
 	gboolean block_matched = FALSE;
 
 	g_return_if_fail(param != NULL);
 	g_return_if_fail(label != NULL);
 	g_return_if_fail(rcfile != NULL);
 
-	rcpath = g_strconcat(g_get_home_dir(), G_DIR_SEPARATOR_S, RC_DIR,
-			     G_DIR_SEPARATOR_S, rcfile, NULL);
+	rcpath = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, rcfile, NULL);
 	if ((orig_fp = fopen(rcpath, "r")) == NULL) {
-		fprintf(stderr, "%s: ", rcpath);
-		perror("fopen");
+		if (ENOENT != errno) FILE_OP_ERROR(rcpath, "fopen");
 	}
 
-	tmprcpath = g_strconcat(g_get_tmp_dir(), G_DIR_SEPARATOR_S, ".rctmp",
-				NULL);
-	if ((tmp_fp = fopen(tmprcpath, "w")) == NULL) {
-		fprintf(stderr, "%s: ", tmprcpath);
-		perror("fopen");
+	if ((pfile = prefs_write_open(rcpath)) == NULL) {
+		g_warning(_("failed to write configuration to file\n"));
 		if (orig_fp) fclose(orig_fp);
-		g_free(tmprcpath);
 		g_free(rcpath);
 		return;
 	}
 
-	if (chmod(tmprcpath, S_IRUSR|S_IWUSR) < 0) {
-		fprintf(stderr, "%s: ", tmprcpath);
-		perror("chmod");
-	}
-
 	block_label = g_strdup_printf("[%s]", label);
 
 	/* search aiming block */
@@ -165,78 +182,171 @@ void prefs_save_config(PrefParam *param, const gchar *label,
 				block_matched = TRUE;
 				break;
 			} else
-				fputs(buf, tmp_fp);
+				TRY(fputs(buf, pfile->fp) != EOF);
 		}
 	}
 
-	fprintf(tmp_fp, "%s\n", block_label);
+	TRY(fprintf(pfile->fp, "%s\n", block_label) > 0);
 	g_free(block_label);
+	block_label = NULL;
 
 	/* write all param data to file */
-	prefs_write_param(param, tmp_fp);
+	TRY(prefs_write_param(param, pfile->fp) == 0);
 
 	if (block_matched) {
 		while (fgets(buf, sizeof(buf), orig_fp) != NULL) {
 			/* next block */
 			if (buf[0] == '[') {
-				fputc('\n', tmp_fp);
-				fputs(buf, tmp_fp);
+				TRY(fputc('\n', pfile->fp) != EOF &&
+				    fputs(buf, pfile->fp)  != EOF);
 				break;
 			}
 		}
-		while (fgets(buf, sizeof(buf), orig_fp) != NULL) {
-			fputs(buf, tmp_fp);
-		}
+		while (fgets(buf, sizeof(buf), orig_fp) != NULL)
+			TRY(fputs(buf, pfile->fp) != EOF);
 	}
-	fclose(tmp_fp);
+
 	if (orig_fp) fclose(orig_fp);
-
-	if (copy_file(tmprcpath, rcpath) < 0)
-		g_warning(_("unable to copy file %s to %s\n"),
-			  tmprcpath, rcpath);
-	else {
-		if (unlink(tmprcpath) < 0) {
-			fprintf(stderr, "%s :", tmprcpath);
-			perror("unlink");
-		}
-	}
-
-	g_free(tmprcpath);
+	if (prefs_write_close(pfile) < 0)
+		g_warning(_("failed to write configuration to file\n"));
 	g_free(rcpath);
 
 	debug_print(_("Configuration is saved.\n"));
 }
 
-void prefs_write_param(PrefParam *param, FILE *fp)
+gint prefs_write_param(PrefParam *param, FILE *fp)
 {
 	gint i;
+	gchar buf[PREFSBUFSIZE];
 
 	for (i = 0; param[i].name != NULL; i++) {
 		switch (param[i].type) {
 		case P_STRING:
-			fprintf(fp, "%s=%s\n", param[i].name,
-				*((gchar **)param[i].data) ?
-				*((gchar **)param[i].data) : "");
+			g_snprintf(buf, sizeof(buf), "%s=%s\n", param[i].name,
+				   *((gchar **)param[i].data) ?
+				   *((gchar **)param[i].data) : "");
 			break;
 		case P_INT:
-			fprintf(fp, "%s=%d\n", param[i].name,
-				*((gint *)param[i].data));
+			g_snprintf(buf, sizeof(buf), "%s=%d\n", param[i].name,
+				   *((gint *)param[i].data));
 			break;
 		case P_BOOL:
-			fprintf(fp, "%s=%d\n", param[i].name,
-				*((gboolean *)param[i].data));
+			g_snprintf(buf, sizeof(buf), "%s=%d\n", param[i].name,
+				   *((gboolean *)param[i].data));
 			break;
 		case P_ENUM:
-			fprintf(fp, "%s=%d\n", param[i].name,
-				*((DummyEnum *)param[i].data));
+			g_snprintf(buf, sizeof(buf), "%s=%d\n", param[i].name,
+				   *((DummyEnum *)param[i].data));
 			break;
 		case P_USHORT:
-			fprintf(fp, "%s=%d\n", param[i].name,
-				*((gushort *)param[i].data));
+			g_snprintf(buf, sizeof(buf), "%s=%d\n", param[i].name,
+				   *((gushort *)param[i].data));
 			break;
 		default:
+			buf[0] = '\0';
+		}
+
+		if (buf[0] != '\0') {
+			if (fputs(buf, fp) == EOF) {
+				perror("fputs");
+				return -1;
+			}
 		}
 	}
+
+	return 0;
+}
+
+PrefFile *prefs_write_open(const gchar *path)
+{
+	PrefFile *pfile;
+	gchar *tmppath;
+	FILE *fp;
+
+	g_return_val_if_fail(path != NULL, NULL);
+
+	tmppath = g_strconcat(path, ".tmp", NULL);
+	if ((fp = fopen(tmppath, "w")) == NULL) {
+		FILE_OP_ERROR(tmppath, "fopen");
+		g_free(tmppath);
+		return NULL;
+	}
+
+	if (chmod(tmppath, S_IRUSR|S_IWUSR) < 0)
+		FILE_OP_ERROR(tmppath, "chmod");
+
+	g_free(tmppath);
+
+	pfile = g_new(PrefFile, 1);
+	pfile->fp = fp;
+	pfile->path = g_strdup(path);
+
+	return pfile;
+}
+
+gint prefs_write_close(PrefFile *pfile)
+{
+	FILE *fp;
+	gchar *path;
+	gchar *tmppath;
+	gchar *bakpath = NULL;
+
+	g_return_val_if_fail(pfile != NULL, -1);
+
+	fp = pfile->fp;
+	path = pfile->path;
+	g_free(pfile);
+
+	tmppath = g_strconcat(path, ".tmp", NULL);
+	if (fclose(fp) == EOF) {
+		FILE_OP_ERROR(tmppath, "fclose");
+		unlink(tmppath);
+		g_free(path);
+		g_free(tmppath);
+		return -1;
+	}
+
+	if (is_file_exist(path)) {
+		bakpath = g_strconcat(path, ".bak", NULL);
+		if (rename(path, bakpath) < 0) {
+			FILE_OP_ERROR(path, "rename");
+			unlink(tmppath);
+			g_free(path);
+			g_free(tmppath);
+			g_free(bakpath);
+			return -1;
+		}
+	}
+
+	if (rename(tmppath, path) < 0) {
+		FILE_OP_ERROR(tmppath, "rename");
+		unlink(tmppath);
+		g_free(path);
+		g_free(tmppath);
+		g_free(bakpath);
+		return -1;
+	}
+
+	g_free(path);
+	g_free(tmppath);
+	g_free(bakpath);
+	return 0;
+}
+
+gint prefs_write_close_revert(PrefFile *pfile)
+{
+	gchar *tmppath;
+
+	g_return_val_if_fail(pfile != NULL, -1);
+
+	tmppath = g_strconcat(pfile->path, ".tmp", NULL);
+	fclose(pfile->fp);
+	if (unlink(tmppath) < 0) FILE_OP_ERROR(tmppath, "unlink");
+	g_free(tmppath);
+	g_free(pfile->path);
+	g_free(pfile);
+
+	return 0;
 }
 
 void prefs_set_default(PrefParam *param)
@@ -336,10 +446,9 @@ void prefs_dialog_create(PrefsDialog *dialog)
 
 	window = gtk_window_new (GTK_WINDOW_DIALOG);
 	gtk_container_set_border_width (GTK_CONTAINER (window), 8);
-	gtk_widget_set_usize(window, 450, 450);
 	gtk_window_position (GTK_WINDOW(window), GTK_WIN_POS_CENTER);
 	gtk_window_set_modal (GTK_WINDOW (window), TRUE);
-	gtk_window_set_policy (GTK_WINDOW(window), FALSE, FALSE, FALSE);
+	gtk_window_set_policy (GTK_WINDOW(window), FALSE, TRUE, FALSE);
 
 	vbox = gtk_vbox_new (FALSE, 6);
 	gtk_widget_show(vbox);
@@ -539,7 +648,9 @@ void prefs_set_data_from_text(PrefParam *pparam)
 			g_free(text);
 			break;
 		}
-		tmpp = tmp = g_malloc(strlen(text) * 2 + 1);
+
+		Xalloca(tmpp = tmp, strlen(text) * 2 + 1,
+			{ *str = NULL; break; });
 		while (*tp) {
 			if (*tp == '\n') {
 				*tmpp++ = '\\';
@@ -550,7 +661,6 @@ void prefs_set_data_from_text(PrefParam *pparam)
 		}
 		*tmpp = '\0';
 		*str = g_strdup(tmp);
-		g_free(tmp);
 		g_free(text);
 		break;
 	default:
@@ -571,25 +681,27 @@ void prefs_set_text(PrefParam *pparam)
 	case P_STRING:
 		str = (gchar **)pparam->data;
 		if (*str) {
-			bufp = buf = g_malloc(strlen(*str) + 1);
-			sp = *str;
-			while (*sp) {
-				if (*sp == '\\' && *(sp + 1) == 'n') {
-					*bufp++ = '\n';
-					sp += 2;
-				} else
-					*bufp++ = *sp++;
+			bufp = buf = alloca(strlen(*str) + 1);
+			if (!buf) buf = "";
+			else {
+				sp = *str;
+				while (*sp) {
+					if (*sp == '\\' && *(sp + 1) == 'n') {
+						*bufp++ = '\n';
+						sp += 2;
+					} else
+						*bufp++ = *sp++;
+				}
+				*bufp = '\0';
 			}
-			*bufp = '\0';
 		} else
-			buf = g_strdup("");
+			buf = "";
 
 		text = GTK_TEXT(*pparam->widget);
 		gtk_text_set_point(text, 0);
 		gtk_text_forward_delete(text, gtk_text_get_length(text));
 		gtk_text_set_point(text, 0);
 		gtk_text_insert(text, NULL, NULL, NULL, buf, -1);
-		g_free(buf);
 		break;
 	default:
 		g_warning("Invalid PrefType for GtkText widget: %d\n",
