@@ -1,9 +1,28 @@
-/* prefs_account.c */
+/*
+ * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
+ * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
 
+#include "defs.h"
+
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <stdio.h>
@@ -11,6 +30,7 @@
 #include <sys/stat.h>
 #include <string.h>
 #include <stdlib.h>
+#include <ctype.h>
 
 #include "intl.h"
 #include "main.h"
@@ -19,6 +39,7 @@
 #include "account.h"
 #include "mainwindow.h"
 #include "manage_window.h"
+#include "menu.h"
 #include "gtkutils.h"
 #include "utils.h"
 #include "alertpanel.h"
@@ -37,19 +58,28 @@ static struct Basic {
 	GtkWidget *addr_entry;
 	GtkWidget *org_entry;
 
+	GtkWidget *serv_table;
 	GtkWidget *protocol_optmenu;
+	GtkWidget *inbox_label;
 	GtkWidget *inbox_entry;
+	GtkWidget *recvserv_label;
+	GtkWidget *smtpserv_label;
+	GtkWidget *nntpserv_label;
 	GtkWidget *recvserv_entry;
 	GtkWidget *smtpserv_entry;
+	GtkWidget *nntpserv_entry;
+	GtkWidget *uid_label;
+	GtkWidget *pass_label;
 	GtkWidget *uid_entry;
 	GtkWidget *pass_entry;
 } basic;
 
 static struct Receive {
+	GtkWidget *pop3_frame;
 	GtkWidget *rmmail_chkbtn;
 	GtkWidget *getall_chkbtn;
-	GtkWidget *autochk_chkbtn;
-	GtkWidget *assort_on_recv_chkbtn;
+	GtkWidget *recvatgetall_chkbtn;
+	GtkWidget *filter_on_recv_chkbtn;
 } receive;
 
 static struct Send {
@@ -57,6 +87,13 @@ static struct Send {
 	GtkWidget *msgid_chkbtn;
 
 	GtkWidget *customhdr_chkbtn;
+
+	GtkWidget *autocc_chkbtn;
+	GtkWidget *autocc_entry;
+	GtkWidget *autobcc_chkbtn;
+	GtkWidget *autobcc_entry;
+	GtkWidget *autoreplyto_chkbtn;
+	GtkWidget *autoreplyto_entry;
 } send;
 
 static struct Compose {
@@ -74,6 +111,7 @@ static struct Advanced {
 
 static void prefs_account_protocol_set_data_from_optmenu(PrefParam *pparam);
 static void prefs_account_protocol_set_optmenu		(PrefParam *pparam);
+static void prefs_account_protocol_activated		(GtkMenuItem *menuitem);
 
 static PrefParam param[] = {
 	/* Basic */
@@ -93,7 +131,7 @@ static PrefParam param[] = {
 	{"organization", NULL, &tmp_ac_prefs.organization, P_STRING,
 	 &basic.org_entry, prefs_set_data_from_entry, prefs_set_entry},
 
-	{"protocol", NULL, &tmp_ac_prefs.protocol, P_OTHER,
+	{"protocol", NULL, &tmp_ac_prefs.protocol, P_ENUM,
 	 &basic.protocol_optmenu,
 	 prefs_account_protocol_set_data_from_optmenu,
 	 prefs_account_protocol_set_optmenu},
@@ -101,9 +139,18 @@ static PrefParam param[] = {
 	{"receive_server", NULL, &tmp_ac_prefs.recv_server, P_STRING,
 	 &basic.recvserv_entry, prefs_set_data_from_entry, prefs_set_entry},
 
-	{"smtp_server", "localhost", &tmp_ac_prefs.smtp_server, P_STRING,
+	{"smtp_server", NULL, &tmp_ac_prefs.smtp_server, P_STRING,
 	 &basic.smtpserv_entry, prefs_set_data_from_entry, prefs_set_entry},
 
+	{"nntp_server", NULL, &tmp_ac_prefs.nntp_server, P_STRING,
+	 &basic.nntpserv_entry, prefs_set_data_from_entry, prefs_set_entry},
+
+	{"use_smtp_auth", "FALSE", &tmp_ac_prefs.use_smtp_auth, P_BOOL,
+	 NULL, NULL, NULL},
+
+	{"use_nntp_auth", "FALSE", &tmp_ac_prefs.use_nntp_auth, P_BOOL,
+	 NULL, NULL, NULL},
+
 	{"user_id", "ENV_USER", &tmp_ac_prefs.userid, P_STRING,
 	 &basic.uid_entry, prefs_set_data_from_entry, prefs_set_entry},
 
@@ -122,12 +169,12 @@ static PrefParam param[] = {
 	 &receive.getall_chkbtn,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
 
-	{"autochk_newmail", "FALSE", &tmp_ac_prefs.autochk, P_BOOL,
-	 &receive.autochk_chkbtn,
+	{"receive_at_get_all", "TRUE", &tmp_ac_prefs.recv_at_getall, P_BOOL,
+	 &receive.recvatgetall_chkbtn,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
 
-	{"assort_on_receive", "FALSE", &tmp_ac_prefs.assort_on_recv, P_BOOL,
-	 &receive.assort_on_recv_chkbtn,
+	{"filter_on_receive", "FALSE", &tmp_ac_prefs.filter_on_recv, P_BOOL,
+	 &receive.filter_on_recv_chkbtn,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
 
 	/* Send */
@@ -135,7 +182,7 @@ static PrefParam param[] = {
 	 &send.date_chkbtn,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
 
-	{"generate_msgid", "FALSE", &tmp_ac_prefs.gen_msgid, P_BOOL,
+	{"generate_msgid", "TRUE", &tmp_ac_prefs.gen_msgid, P_BOOL,
 	 &send.msgid_chkbtn,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
 
@@ -143,6 +190,30 @@ static PrefParam param[] = {
 	 &send.customhdr_chkbtn,
 	 prefs_set_data_from_toggle, prefs_set_toggle},
 
+	{"set_autocc", "FALSE", &tmp_ac_prefs.set_autocc, P_BOOL,
+	 &send.autocc_chkbtn,
+	 prefs_set_data_from_toggle, prefs_set_toggle},
+
+	{"auto_cc", NULL, &tmp_ac_prefs.auto_cc, P_STRING,
+	 &send.autocc_entry,
+	 prefs_set_data_from_entry, prefs_set_entry},
+
+	{"set_autobcc", "FALSE", &tmp_ac_prefs.set_autobcc, P_BOOL,
+	 &send.autobcc_chkbtn,
+	 prefs_set_data_from_toggle, prefs_set_toggle},
+
+	{"auto_bcc", NULL, &tmp_ac_prefs.auto_bcc, P_STRING,
+	 &send.autobcc_entry,
+	 prefs_set_data_from_entry, prefs_set_entry},
+
+	{"set_autoreplyto", "FALSE", &tmp_ac_prefs.set_autoreplyto, P_BOOL,
+	 &send.autoreplyto_chkbtn,
+	 prefs_set_data_from_toggle, prefs_set_toggle},
+
+	{"auto_replyto", NULL, &tmp_ac_prefs.auto_replyto, P_STRING,
+	 &send.autoreplyto_entry,
+	 prefs_set_data_from_entry, prefs_set_entry},
+
 	/* Compose */
 	{"signature_path", "~/"DEFAULT_SIGNATURE, &tmp_ac_prefs.sig_path, P_STRING,
 	 &compose.sigpath_entry,
@@ -191,26 +262,34 @@ static gint prefs_account_apply			(void);
 static void prefs_account_cancel		(void);
 
 #define VSPACING	12
+#define VSPACING_NARROW	6
 #define BOX_BORDER	16
 
 void prefs_account_read_config(PrefsAccount *ac_prefs, const gchar *label)
 {
+	const gchar *p = label;
+	gint id;
+
 	g_return_if_fail(ac_prefs != NULL);
 	g_return_if_fail(label != NULL);
 
 	memset(&tmp_ac_prefs, 0, sizeof(PrefsAccount));
 	prefs_read_config(param, label, ACCOUNT_RC);
 	*ac_prefs = tmp_ac_prefs;
+	while (*p && !isdigit(*p)) p++;
+	id = atoi(p);
+	if (id < 0) g_warning("wrong account id: %d\n", id);
+	ac_prefs->account_id = id;
 }
 
-void prefs_account_save_config(PrefsAccount *ac_prefs, gint id)
+void prefs_account_save_config(PrefsAccount *ac_prefs)
 {
 	gchar *buf;
 
 	g_return_if_fail(ac_prefs != NULL);
 
 	tmp_ac_prefs = *ac_prefs;
-	buf = g_strdup_printf("Account: %d", id);
+	buf = g_strdup_printf("Account: %d", ac_prefs->account_id);
 	prefs_save_config(param, buf, ACCOUNT_RC);
 }
 
@@ -218,47 +297,37 @@ void prefs_account_save_config_all(GList *account_list)
 {
 	GList *cur;
 	gchar *rcpath;
-	FILE *fp;
-	gint id = 0;
+	PrefFile *pfile;
 
 	if (!account_list) return;
 
-	rcpath = g_strconcat(g_get_home_dir(), G_DIR_SEPARATOR_S, RC_DIR,
-			     G_DIR_SEPARATOR_S, ACCOUNT_RC, NULL);
-
-	/* backup previous rc file */
-	if (is_file_exist(rcpath)) {
-		gchar *rcpath_bak;
-
-		rcpath_bak = g_strconcat(rcpath, ".bak", NULL);
-		if (rename(rcpath, rcpath_bak) < 0) {
-			fprintf(stderr, "%s: ", rcpath);
-			perror("rename");
-		}
-		g_free(rcpath_bak);
-	}
-
-	if ((fp = fopen(rcpath, "w")) == NULL) {
-		fprintf(stderr, "%s: ", rcpath);
-		perror("fopen");
+	rcpath = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S, ACCOUNT_RC, NULL);
+	if ((pfile = prefs_write_open(rcpath)) == NULL) {
 		g_free(rcpath);
 		return;
 	}
-	if (chmod(rcpath, S_IRUSR|S_IWUSR) < 0) {
-		fprintf(stderr, "%s: ", rcpath);
-		perror("chmod");
-	}
 	g_free(rcpath);
 
 	for (cur = account_list; cur != NULL; cur = cur->next) {
 		tmp_ac_prefs = *(PrefsAccount *)cur->data;
-		fprintf(fp, "[Account: %d]\n", id++);
-		prefs_write_param(param, fp);
-		if (cur->next)
-			fputc('\n', fp);
+		if (fprintf(pfile->fp, "[Account: %d]\n",
+			    tmp_ac_prefs.account_id) <= 0 ||
+		    prefs_write_param(param, pfile->fp) < 0) {
+			g_warning(_("failed to write configuration to file\n"));
+			prefs_write_close_revert(pfile);
+			return;
+		}
+		if (cur->next) {
+			if (fputc('\n', pfile->fp) == EOF) {
+				FILE_OP_ERROR(rcpath, "fputc");
+				prefs_write_close_revert(pfile);
+				return;
+			}
+		}
 	}
 
-	fclose(fp);
+	if (prefs_write_close(pfile) < 0)
+		g_warning(_("failed to write configuration to file\n"));
 }
 
 void prefs_account_free(PrefsAccount *ac_prefs)
@@ -269,9 +338,24 @@ void prefs_account_free(PrefsAccount *ac_prefs)
 	prefs_free(param);
 }
 
+static gint prefs_account_get_new_id(void)
+{
+	GList *ac_list;
+	PrefsAccount *ac;
+	static gint last_id = 0;
+
+	for (ac_list = account_get_list(); ac_list != NULL;
+	     ac_list = ac_list->next) {
+		ac = (PrefsAccount *)ac_list->data;
+		if (last_id < ac->account_id)
+			last_id = ac->account_id;
+	}
+
+	return last_id + 1;
+}
+
 PrefsAccount *prefs_account_open(PrefsAccount *ac_prefs)
 {
-	static gint ac_id = 1;
 	gboolean new_account = FALSE;
 
 	debug_print(_("Opening account preferences window...\n"));
@@ -280,6 +364,7 @@ PrefsAccount *prefs_account_open(PrefsAccount *ac_prefs)
 
 	if (!ac_prefs) {
 		ac_prefs = g_new0(PrefsAccount, 1);
+		ac_prefs->account_id = prefs_account_get_new_id();
 		new_account = TRUE;
 	}
 
@@ -294,12 +379,22 @@ PrefsAccount *prefs_account_open(PrefsAccount *ac_prefs)
 	tmp_ac_prefs = *ac_prefs;
 
 	if (new_account) {
+		PrefsAccount *def_ac;
 		gchar *buf;
 
 		prefs_set_dialog_to_default(param);
-		buf = g_strdup_printf(_("Account%d"), ac_id);
+		buf = g_strdup_printf(_("Account%d"), ac_prefs->account_id);
 		gtk_entry_set_text(GTK_ENTRY(basic.acname_entry), buf);
 		g_free(buf);
+		def_ac = account_get_default();
+		if (def_ac) {
+			gtk_entry_set_text(GTK_ENTRY(basic.name_entry),
+					   def_ac->name ? def_ac->name : "");
+			gtk_entry_set_text(GTK_ENTRY(basic.addr_entry),
+					   def_ac->address ? def_ac->address : "");
+			gtk_entry_set_text(GTK_ENTRY(basic.org_entry),
+					   def_ac->organization ? def_ac->organization : "");
+		}
 		gtk_window_set_title(GTK_WINDOW(dialog.window),
 				     _("Preferences for new account"));
 		gtk_widget_hide(dialog.apply_btn);
@@ -319,7 +414,6 @@ PrefsAccount *prefs_account_open(PrefsAccount *ac_prefs)
 		return NULL;
 	} else {
 		*ac_prefs = tmp_ac_prefs;
-		if (new_account) ac_id++;
 		return ac_prefs;
 	}
 }
@@ -339,6 +433,8 @@ static void prefs_account_create(void)
 			   GTK_SIGNAL_FUNC(prefs_account_key_pressed), NULL);
 	gtk_signal_connect(GTK_OBJECT(dialog.window), "focus_in_event",
 			   GTK_SIGNAL_FUNC(manage_window_focus_in), NULL);
+	gtk_signal_connect(GTK_OBJECT(dialog.window), "focus_out_event",
+			   GTK_SIGNAL_FUNC(manage_window_focus_out), NULL);
 	gtk_signal_connect(GTK_OBJECT(dialog.ok_btn), "clicked",
 	 		   GTK_SIGNAL_FUNC(prefs_account_ok), NULL);
 	gtk_signal_connect(GTK_OBJECT(dialog.apply_btn), "clicked",
@@ -365,18 +461,18 @@ static void prefs_account_create(void)
 
 		width = gtkut_get_font_width(style->font);
 		height = gtkut_get_font_height(style->font);
-		gtk_widget_set_usize(dialog.window, width * 74, height * 32);
+		gtk_widget_set_usize
+			(dialog.notebook,
+			 width * 58 + BOX_BORDER * 2 + 24,
+			 height * 10 + BOX_BORDER * 2 + VSPACING * 15 + 64);
 	}
 }
 
-#define MENUITEM_ADD(menu, menuitem, label, data) \
+#define SET_ACTIVATE(menuitem) \
 { \
-	menuitem = gtk_menu_item_new_with_label(label); \
-	gtk_widget_show(menuitem); \
-	gtk_menu_append(GTK_MENU(menu), menuitem); \
-	if (data) \
-		gtk_object_set_user_data(GTK_OBJECT(menuitem), \
-					 GINT_TO_POINTER(data)); \
+	gtk_signal_connect(GTK_OBJECT(menuitem), "activate", \
+			   GTK_SIGNAL_FUNC(prefs_account_protocol_activated), \
+			   NULL); \
 }
 
 static void prefs_account_basic_create(void)
@@ -397,14 +493,21 @@ static void prefs_account_basic_create(void)
 	GtkWidget *optmenu;
 	GtkWidget *optmenu_menu;
 	GtkWidget *menuitem;
+	GtkWidget *inbox_label;
 	GtkWidget *inbox_entry;
-	GtkWidget *table2;
+	GtkWidget *serv_table;
+	GtkWidget *recvserv_label;
+	GtkWidget *smtpserv_label;
+	GtkWidget *nntpserv_label;
 	GtkWidget *recvserv_entry;
 	GtkWidget *smtpserv_entry;
+	GtkWidget *nntpserv_entry;
+	GtkWidget *uid_label;
+	GtkWidget *pass_label;
 	GtkWidget *uid_entry;
 	GtkWidget *pass_entry;
 
-	vbox1 = gtk_vbox_new (FALSE, 8);
+	vbox1 = gtk_vbox_new (FALSE, VSPACING);
 	gtk_widget_show (vbox1);
 	gtk_container_add (GTK_CONTAINER (dialog.notebook), vbox1);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox1), BOX_BORDER);
@@ -425,17 +528,13 @@ static void prefs_account_basic_create(void)
 	gtk_widget_show (default_chkbtn);
 	gtk_box_pack_end (GTK_BOX (hbox), default_chkbtn, FALSE, FALSE, 0);
 
-	frame1 = gtk_frame_new (_("Personal information"));
-	gtk_widget_show (frame1);
-	gtk_box_pack_start (GTK_BOX (vbox1), frame1, FALSE, FALSE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (frame1), 4);
-	gtk_frame_set_label_align (GTK_FRAME (frame1), 0.01, 0.5);
+	PACK_FRAME (vbox1, frame1, _("Personal information"));
 
 	table1 = gtk_table_new (3, 2, FALSE);
 	gtk_widget_show (table1);
 	gtk_container_add (GTK_CONTAINER (frame1), table1);
 	gtk_container_set_border_width (GTK_CONTAINER (table1), 8);
-	gtk_table_set_row_spacings (GTK_TABLE (table1), 8);
+	gtk_table_set_row_spacings (GTK_TABLE (table1), VSPACING_NARROW);
 	gtk_table_set_col_spacings (GTK_TABLE (table1), 8);
 
 	label = gtk_label_new (_("Full name"));
@@ -474,13 +573,9 @@ static void prefs_account_basic_create(void)
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
 
-	frame2 = gtk_frame_new (_("Server information"));
-	gtk_widget_show (frame2);
-	gtk_box_pack_start (GTK_BOX (vbox1), frame2, FALSE, FALSE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (frame2), 4);
-	gtk_frame_set_label_align (GTK_FRAME (frame2), 0.01, 0.5);
+	PACK_FRAME (vbox1, frame2, _("Server information"));
 
-	vbox2 = gtk_vbox_new (FALSE, 8);
+	vbox2 = gtk_vbox_new (FALSE, VSPACING_NARROW);
 	gtk_widget_show (vbox2);
 	gtk_container_add (GTK_CONTAINER (frame2), vbox2);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox2), 8);
@@ -499,73 +594,93 @@ static void prefs_account_basic_create(void)
 
 	optmenu_menu = gtk_menu_new ();
 
-	MENUITEM_ADD (optmenu_menu, menuitem, _("POP3 (normal)"),  A_POP3 )
-	MENUITEM_ADD (optmenu_menu, menuitem, _("POP3 (APOP auth)"),  A_APOP )
-	MENUITEM_ADD (optmenu_menu, menuitem, _("IMAP4"), A_IMAP4)
-	gtk_widget_set_sensitive(menuitem, FALSE);
+	MENUITEM_ADD (optmenu_menu, menuitem, _("POP3 (normal)"),  A_POP3);
+	SET_ACTIVATE (menuitem);
+	MENUITEM_ADD (optmenu_menu, menuitem, _("POP3 (APOP auth)"),  A_APOP);
+	SET_ACTIVATE (menuitem);
+	MENUITEM_ADD (optmenu_menu, menuitem, _("IMAP4"), A_IMAP4);
+	SET_ACTIVATE (menuitem);
+	MENUITEM_ADD (optmenu_menu, menuitem, _("News (NNTP)"), A_NNTP);
+	SET_ACTIVATE (menuitem);
+	MENUITEM_ADD (optmenu_menu, menuitem, _("None (local)"), A_LOCAL);
+	SET_ACTIVATE (menuitem);
 
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (optmenu), optmenu_menu);
 
-	label = gtk_label_new (_("Inbox"));
-	gtk_widget_show (label);
-	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+	inbox_label = gtk_label_new (_("Inbox"));
+	gtk_widget_show (inbox_label);
+	gtk_box_pack_start (GTK_BOX (hbox), inbox_label, FALSE, FALSE, 0);
 
 	inbox_entry = gtk_entry_new ();
 	gtk_widget_show (inbox_entry);
 	gtk_box_pack_start (GTK_BOX (hbox), inbox_entry, TRUE, TRUE, 0);
 
-	table2 = gtk_table_new (3, 4, FALSE);
-	gtk_widget_show (table2);
-	gtk_box_pack_start (GTK_BOX (vbox2), table2, FALSE, FALSE, 0);
-	gtk_table_set_row_spacings (GTK_TABLE (table2), 8);
-	gtk_table_set_col_spacings (GTK_TABLE (table2), 8);
+	serv_table = gtk_table_new (5, 4, FALSE);
+	gtk_widget_show (serv_table);
+	gtk_box_pack_start (GTK_BOX (vbox2), serv_table, FALSE, FALSE, 0);
+	gtk_table_set_row_spacings (GTK_TABLE (serv_table), VSPACING_NARROW);
+	gtk_table_set_col_spacings (GTK_TABLE (serv_table), 8);
+
+	nntpserv_entry = gtk_entry_new ();
+	gtk_widget_show (nntpserv_entry);
+	gtk_table_attach (GTK_TABLE (serv_table), nntpserv_entry, 1, 4, 0, 1,
+			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
+			  GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
+	gtk_table_set_row_spacing (GTK_TABLE (serv_table), 0, 0);
 
 	recvserv_entry = gtk_entry_new ();
 	gtk_widget_show (recvserv_entry);
-	gtk_table_attach (GTK_TABLE (table2), recvserv_entry, 1, 4, 0, 1,
+	gtk_table_attach (GTK_TABLE (serv_table), recvserv_entry, 1, 4, 1, 2,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
 
 	smtpserv_entry = gtk_entry_new ();
 	gtk_widget_show (smtpserv_entry);
-	gtk_table_attach (GTK_TABLE (table2), smtpserv_entry, 1, 4, 1, 2,
+	gtk_table_attach (GTK_TABLE (serv_table), smtpserv_entry, 1, 4, 2, 3,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
 
 	uid_entry = gtk_entry_new ();
 	gtk_widget_show (uid_entry);
-	gtk_table_attach (GTK_TABLE (table2), uid_entry, 1, 2, 2, 3,
+	gtk_table_attach (GTK_TABLE (serv_table), uid_entry, 1, 2, 4, 5,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
 
 	pass_entry = gtk_entry_new ();
 	gtk_widget_show (pass_entry);
-	gtk_table_attach (GTK_TABLE (table2), pass_entry, 3, 4, 2, 3,
+	gtk_table_attach (GTK_TABLE (serv_table), pass_entry, 3, 4, 4, 5,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
 			  GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
 	gtk_entry_set_visibility (GTK_ENTRY (pass_entry), FALSE);
 
-	label = gtk_label_new (_("Server for receiving"));
-	gtk_widget_show (label);
-	gtk_table_attach (GTK_TABLE (table2), label, 0, 1, 0, 1,
+	nntpserv_label = gtk_label_new (_("News server"));
+	gtk_widget_show (nntpserv_label);
+	gtk_table_attach (GTK_TABLE (serv_table), nntpserv_label, 0, 1, 0, 1,
 			  GTK_FILL, 0, 0, 0);
-	gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
+	gtk_misc_set_alignment (GTK_MISC (nntpserv_label), 1, 0.5);
 
-	label = gtk_label_new (_("SMTP server (send)"));
-	gtk_widget_show (label);
-	gtk_table_attach (GTK_TABLE (table2), label, 0, 1, 1, 2,
+	recvserv_label = gtk_label_new (_("Server for receiving"));
+	gtk_widget_show (recvserv_label);
+	gtk_table_attach (GTK_TABLE (serv_table), recvserv_label, 0, 1, 1, 2,
 			  GTK_FILL, 0, 0, 0);
-	gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
+	gtk_misc_set_alignment (GTK_MISC (recvserv_label), 1, 0.5);
 
-	label = gtk_label_new (_("User ID"));
-	gtk_widget_show (label);
-	gtk_table_attach (GTK_TABLE (table2), label, 0, 1, 2, 3,
+	smtpserv_label = gtk_label_new (_("SMTP server (send)"));
+	gtk_widget_show (smtpserv_label);
+	gtk_table_attach (GTK_TABLE (serv_table), smtpserv_label, 0, 1, 2, 3,
 			  GTK_FILL, 0, 0, 0);
-	gtk_misc_set_alignment (GTK_MISC (label), 1, 0.5);
+	gtk_misc_set_alignment (GTK_MISC (smtpserv_label), 1, 0.5);
+	gtk_table_set_row_spacing (GTK_TABLE (serv_table), 2, 0);
 
-	label = gtk_label_new (_("Password"));
-	gtk_widget_show (label);
-	gtk_table_attach (GTK_TABLE (table2), label, 2, 3, 2, 3,
+	uid_label = gtk_label_new (_("User ID"));
+	gtk_widget_show (uid_label);
+	gtk_table_attach (GTK_TABLE (serv_table), uid_label, 0, 1, 4, 5,
+			  GTK_FILL, 0, 0, 0);
+	gtk_misc_set_alignment (GTK_MISC (uid_label), 1, 0.5);
+
+	pass_label = gtk_label_new (_("Password"));
+	gtk_widget_show (pass_label);
+	gtk_table_attach (GTK_TABLE (serv_table), pass_label, 2, 3, 4, 5,
 			  0, 0, 0, 0);
 
 	basic.acname_entry   = acname_entry;
@@ -575,92 +690,93 @@ static void prefs_account_basic_create(void)
 	basic.addr_entry = addr_entry;
 	basic.org_entry  = org_entry;
 
+	basic.serv_table       = serv_table;
 	basic.protocol_optmenu = optmenu;
+	basic.inbox_label      = inbox_label;
 	basic.inbox_entry      = inbox_entry;
+	basic.recvserv_label   = recvserv_label;
 	basic.recvserv_entry   = recvserv_entry;
+	basic.smtpserv_label   = smtpserv_label;
 	basic.smtpserv_entry   = smtpserv_entry;
+	basic.nntpserv_label   = nntpserv_label;
+	basic.nntpserv_entry   = nntpserv_entry;
+	basic.uid_label        = uid_label;
+	basic.pass_label       = pass_label;
 	basic.uid_entry        = uid_entry;
 	basic.pass_entry       = pass_entry;
 }
 
 static void prefs_account_receive_create(void)
 {
-	GtkWidget *vbox;
+	GtkWidget *vbox1;
+	GtkWidget *frame1;
+	GtkWidget *vbox2;
 	GtkWidget *rmmail_chkbtn;
 	GtkWidget *getall_chkbtn;
-	GtkWidget *autochk_chkbtn;
-	GtkWidget *assort_on_recv_chkbtn;
+	GtkWidget *recvatgetall_chkbtn;
+	GtkWidget *filter_on_recv_chkbtn;
 
-	vbox = gtk_vbox_new (FALSE, VSPACING);
-	gtk_widget_show (vbox);
-	gtk_container_add (GTK_CONTAINER (dialog.notebook), vbox);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), BOX_BORDER);
+	vbox1 = gtk_vbox_new (FALSE, VSPACING);
+	gtk_widget_show (vbox1);
+	gtk_container_add (GTK_CONTAINER (dialog.notebook), vbox1);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox1), BOX_BORDER);
 
-	rmmail_chkbtn = gtk_check_button_new_with_label
-		(_("Remove messages on server when received"));
-	gtk_widget_show (rmmail_chkbtn);
-	gtk_box_pack_start (GTK_BOX (vbox), rmmail_chkbtn, FALSE, FALSE, 0);
+	PACK_FRAME (vbox1, frame1, _("POP3"));
 
-	getall_chkbtn = gtk_check_button_new_with_label
-		(_("Receive all messages on server"));
-	gtk_widget_show (getall_chkbtn);
-	gtk_box_pack_start (GTK_BOX (vbox), getall_chkbtn, FALSE, FALSE, 0);
+	vbox2 = gtk_vbox_new (FALSE, VSPACING_NARROW);
+	gtk_widget_show (vbox2);
+	gtk_container_add (GTK_CONTAINER (frame1), vbox2);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox2), 8);
 
-	autochk_chkbtn =
-		gtk_check_button_new_with_label (_("Auto-check new mail"));
-	gtk_widget_show (autochk_chkbtn);
-	gtk_box_pack_start (GTK_BOX (vbox), autochk_chkbtn, FALSE, FALSE, 0);
-
-	assort_on_recv_chkbtn = gtk_check_button_new_with_label
-		(_("Assort messages on receiving"));
-	gtk_widget_show (assort_on_recv_chkbtn);
-	gtk_box_pack_start (GTK_BOX (vbox), assort_on_recv_chkbtn,
-			    FALSE, FALSE, 0);
-
-	gtk_widget_set_sensitive(autochk_chkbtn, FALSE);
+	PACK_CHECK_BUTTON (vbox2, rmmail_chkbtn,
+			   _("Remove messages on server when received"));
+	PACK_CHECK_BUTTON (vbox2, getall_chkbtn,
+			   _("Receive all messages on server"));
+	PACK_CHECK_BUTTON (vbox2, recvatgetall_chkbtn,
+			   _("Receive at getting from all accounts"));
+	PACK_CHECK_BUTTON (vbox2, filter_on_recv_chkbtn,
+			   _("Filter messages on receiving"));
 
+	receive.pop3_frame            = frame1;
 	receive.rmmail_chkbtn         = rmmail_chkbtn;
 	receive.getall_chkbtn         = getall_chkbtn;
-	receive.autochk_chkbtn        = autochk_chkbtn;
-	receive.assort_on_recv_chkbtn = assort_on_recv_chkbtn;
+	receive.recvatgetall_chkbtn   = recvatgetall_chkbtn;
+	receive.filter_on_recv_chkbtn = filter_on_recv_chkbtn;
 }
 
 static void prefs_account_send_create(void)
 {
-	GtkWidget *vbox;
-	GtkWidget *frame;
+	GtkWidget *vbox1;
 	GtkWidget *vbox2;
+	GtkWidget *frame;
 	GtkWidget *date_chkbtn;
 	GtkWidget *msgid_chkbtn;
 	GtkWidget *hbox;
 	GtkWidget *customhdr_chkbtn;
 	GtkWidget *customhdr_edit_btn;
+	GtkWidget *frame2;
+	GtkWidget *table;
+	GtkWidget *autocc_chkbtn;
+	GtkWidget *autocc_entry;
+	GtkWidget *autobcc_chkbtn;
+	GtkWidget *autobcc_entry;
+	GtkWidget *autoreplyto_chkbtn;
+	GtkWidget *autoreplyto_entry;
 
-	vbox = gtk_vbox_new (FALSE, VSPACING);
-	gtk_widget_show (vbox);
-	gtk_container_add (GTK_CONTAINER (dialog.notebook), vbox);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), BOX_BORDER);
+	vbox1 = gtk_vbox_new (FALSE, VSPACING);
+	gtk_widget_show (vbox1);
+	gtk_container_add (GTK_CONTAINER (dialog.notebook), vbox1);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox1), BOX_BORDER);
 
-	frame = gtk_frame_new (_("Header"));
-	gtk_widget_show (frame);
-	gtk_box_pack_start (GTK_BOX (vbox), frame, FALSE, FALSE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (frame), 4);
-	gtk_frame_set_label_align (GTK_FRAME (frame), 0.01, 0.5);
+	PACK_FRAME (vbox1, frame, _("Header"));
 
-	vbox2 = gtk_vbox_new (FALSE, 8);
+	vbox2 = gtk_vbox_new (FALSE, VSPACING_NARROW);
 	gtk_widget_show (vbox2);
 	gtk_container_add (GTK_CONTAINER (frame), vbox2);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox2), 8);
 
-	date_chkbtn =
-		gtk_check_button_new_with_label (_("Add Date header field"));
-	gtk_widget_show (date_chkbtn);
-	gtk_box_pack_start (GTK_BOX (vbox2), date_chkbtn, FALSE, FALSE, 0);
-
-	msgid_chkbtn =
-		gtk_check_button_new_with_label (_("Generate Message-ID"));
-	gtk_widget_show (msgid_chkbtn);
-	gtk_box_pack_start (GTK_BOX (vbox2), msgid_chkbtn, FALSE, FALSE, 0);
+	PACK_CHECK_BUTTON (vbox2, date_chkbtn, _("Add Date header field"));
+	PACK_CHECK_BUTTON (vbox2, msgid_chkbtn, _("Generate Message-ID"));
 
 	hbox = gtk_hbox_new (FALSE, 12);
 	gtk_widget_show (hbox);
@@ -668,10 +784,8 @@ static void prefs_account_send_create(void)
 
 	gtk_widget_set_sensitive(hbox, FALSE);
 
-	customhdr_chkbtn =
-		gtk_check_button_new_with_label (_("Add user-defined header"));
-	gtk_widget_show (customhdr_chkbtn);
-	gtk_box_pack_start (GTK_BOX (hbox), customhdr_chkbtn, FALSE, FALSE, 0);
+	PACK_CHECK_BUTTON (hbox, customhdr_chkbtn,
+			   _("Add user-defined header"));
 
 	customhdr_edit_btn = gtk_button_new_with_label (_(" Edit... "));
 	gtk_widget_show (customhdr_edit_btn);
@@ -680,26 +794,81 @@ static void prefs_account_send_create(void)
 
 	SET_TOGGLE_SENSITIVITY (customhdr_chkbtn, customhdr_edit_btn);
 
+	PACK_FRAME (vbox1, frame2, _("Automatically set following addresses"));
+
+	table =  gtk_table_new (3, 2, FALSE);
+	gtk_widget_show (table);
+	gtk_container_add (GTK_CONTAINER (frame2), table);
+	gtk_container_set_border_width (GTK_CONTAINER (table), 8);
+	gtk_table_set_row_spacings (GTK_TABLE (table), VSPACING_NARROW);
+	gtk_table_set_col_spacings (GTK_TABLE (table), 8);
+
+	autocc_chkbtn = gtk_check_button_new_with_label (_("Cc"));
+	gtk_widget_show (autocc_chkbtn);
+	gtk_table_attach (GTK_TABLE (table), autocc_chkbtn, 0, 1, 0, 1,
+			  GTK_FILL, 0, 0, 0);
+
+	autocc_entry = gtk_entry_new ();
+	gtk_widget_show (autocc_entry);
+	gtk_table_attach (GTK_TABLE (table), autocc_entry, 1, 2, 0, 1,
+			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
+			  GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
+
+	SET_TOGGLE_SENSITIVITY (autocc_chkbtn, autocc_entry);
+
+	autobcc_chkbtn = gtk_check_button_new_with_label (_("Bcc"));
+	gtk_widget_show (autobcc_chkbtn);
+	gtk_table_attach (GTK_TABLE (table), autobcc_chkbtn, 0, 1, 1, 2,
+			  GTK_FILL, 0, 0, 0);
+
+	autobcc_entry = gtk_entry_new ();
+	gtk_widget_show (autobcc_entry);
+	gtk_table_attach (GTK_TABLE (table), autobcc_entry, 1, 2, 1, 2,
+			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
+			  GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
+
+	SET_TOGGLE_SENSITIVITY (autobcc_chkbtn, autobcc_entry);
+
+	autoreplyto_chkbtn = gtk_check_button_new_with_label (_("Reply-To"));
+	gtk_widget_show (autoreplyto_chkbtn);
+	gtk_table_attach (GTK_TABLE (table), autoreplyto_chkbtn, 0, 1, 2, 3,
+			  GTK_FILL, 0, 0, 0);
+
+	autoreplyto_entry = gtk_entry_new ();
+	gtk_widget_show (autoreplyto_entry);
+	gtk_table_attach (GTK_TABLE (table), autoreplyto_entry, 1, 2, 2, 3,
+			  GTK_EXPAND | GTK_SHRINK | GTK_FILL,
+			  GTK_EXPAND | GTK_SHRINK | GTK_FILL, 0, 0);
+
+	SET_TOGGLE_SENSITIVITY (autoreplyto_chkbtn, autoreplyto_entry);
+
 	send.date_chkbtn      = date_chkbtn;
 	send.msgid_chkbtn     = msgid_chkbtn;
 	send.customhdr_chkbtn = customhdr_chkbtn;
+
+	send.autocc_chkbtn      = autocc_chkbtn;
+	send.autocc_entry       = autocc_entry;
+	send.autobcc_chkbtn     = autobcc_chkbtn;
+	send.autobcc_entry      = autobcc_entry;
+	send.autoreplyto_chkbtn = autoreplyto_chkbtn;
+	send.autoreplyto_entry  = autoreplyto_entry;
 }
 
 static void prefs_account_compose_create(void)
 {
-	GtkWidget *vbox;
+	GtkWidget *vbox1;
 	GtkWidget *hbox;
 	GtkWidget *label;
 	GtkWidget *sigpath_entry;
 
-	vbox = gtk_vbox_new (FALSE, VSPACING);
-	gtk_widget_show (vbox);
-	gtk_container_add (GTK_CONTAINER (dialog.notebook), vbox);
-	gtk_container_set_border_width (GTK_CONTAINER (vbox), BOX_BORDER);
+	vbox1 = gtk_vbox_new (FALSE, VSPACING);
+	gtk_widget_show (vbox1);
+	gtk_container_add (GTK_CONTAINER (dialog.notebook), vbox1);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox1), BOX_BORDER);
 
 	hbox = gtk_hbox_new (FALSE, 8);
 	gtk_widget_show (hbox);
-	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox1), hbox, FALSE, FALSE, 0);
 
 	label = gtk_label_new (_("Signature file"));
 	gtk_widget_show (label);
@@ -715,6 +884,7 @@ static void prefs_account_compose_create(void)
 static void prefs_account_advanced_create(void)
 {
 	GtkWidget *vbox1;
+	GtkWidget *vbox2;
 	GtkWidget *hbox1;
 	GtkWidget *checkbtn_smtpport;
 	GtkWidget *entry_smtpport;
@@ -730,15 +900,15 @@ static void prefs_account_advanced_create(void)
 	gtk_container_add (GTK_CONTAINER (dialog.notebook), vbox1);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox1), BOX_BORDER);
 
+	vbox2 = gtk_vbox_new (FALSE, VSPACING_NARROW);
+	gtk_widget_show (vbox2);
+	gtk_box_pack_start (GTK_BOX (vbox1), vbox2, FALSE, FALSE, 0);
+
 	hbox1 = gtk_hbox_new (FALSE, 8);
 	gtk_widget_show (hbox1);
-	gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox2), hbox1, FALSE, FALSE, 0);
 
-	checkbtn_smtpport =
-		gtk_check_button_new_with_label (_("Specify SMTP port"));
-	gtk_widget_show (checkbtn_smtpport);
-	gtk_box_pack_start (GTK_BOX (hbox1), checkbtn_smtpport,
-			    FALSE, FALSE, 0);
+	PACK_CHECK_BUTTON (hbox1, checkbtn_smtpport, _("Specify SMTP port"));
 
 	entry_smtpport = gtk_entry_new_with_max_length (5);
 	gtk_widget_show (entry_smtpport);
@@ -748,12 +918,9 @@ static void prefs_account_advanced_create(void)
 
 	hbox2 = gtk_hbox_new (FALSE, 8);
 	gtk_widget_show (hbox2);
-	gtk_box_pack_start (GTK_BOX (vbox1), hbox2, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox2), hbox2, FALSE, FALSE, 0);
 
-	checkbtn_popport =
-		gtk_check_button_new_with_label (_("Specify POP3 port"));
-	gtk_widget_show (checkbtn_popport);
-	gtk_box_pack_start (GTK_BOX (hbox2), checkbtn_popport, FALSE, FALSE, 0);
+	PACK_CHECK_BUTTON (hbox2, checkbtn_popport, _("Specify POP3 port"));
 
 	entry_popport = gtk_entry_new_with_max_length (5);
 	gtk_widget_show (entry_popport);
@@ -763,12 +930,9 @@ static void prefs_account_advanced_create(void)
 
 	hbox3 = gtk_hbox_new (FALSE, 8);
 	gtk_widget_show (hbox3);
-	gtk_box_pack_start (GTK_BOX (vbox1), hbox3, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox2), hbox3, FALSE, FALSE, 0);
 
-	checkbtn_domain =
-		gtk_check_button_new_with_label (_("Specify domain name"));
-	gtk_widget_show (checkbtn_domain);
-	gtk_box_pack_start (GTK_BOX (hbox3), checkbtn_domain, FALSE, FALSE, 0);
+	PACK_CHECK_BUTTON (hbox3, checkbtn_domain, _("Specify domain name"));
 
 	entry_domain = gtk_entry_new ();
 	gtk_widget_show (entry_domain);
@@ -798,19 +962,42 @@ static void prefs_account_ok(void)
 
 static gint prefs_account_apply(void)
 {
-	gchar *msg = NULL;
+	RecvProtocol protocol;
+	GtkWidget *menu;
+	GtkWidget *menuitem;
 
-	if (*gtk_entry_get_text(GTK_ENTRY(basic.addr_entry)) == '\0')
-		msg = _("Mail address is not entered.");
-	else if (*gtk_entry_get_text(GTK_ENTRY(basic.recvserv_entry)) == '\0')
-		msg = _("Server for receiving is not entered.");
-	else if (*gtk_entry_get_text(GTK_ENTRY(basic.smtpserv_entry)) == '\0')
-		msg = _("SMTP server is not entered.");
-	else if (*gtk_entry_get_text(GTK_ENTRY(basic.uid_entry)) == '\0')
-		msg = _("User ID is not entered.");
+	menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(basic.protocol_optmenu));
+	menuitem = gtk_menu_get_active(GTK_MENU(menu));
+	protocol = GPOINTER_TO_INT
+		(gtk_object_get_user_data(GTK_OBJECT(menuitem)));
 
-	if (msg) {
-		alertpanel(_("Error"), msg, _("OK"), NULL, NULL);
+	if (*gtk_entry_get_text(GTK_ENTRY(basic.addr_entry)) == '\0') {
+		alertpanel_error(_("Mail address is not entered."));
+		return -1;
+	}
+	if ((protocol == A_POP3 || protocol == A_APOP || protocol == A_LOCAL) &&
+	    *gtk_entry_get_text(GTK_ENTRY(basic.smtpserv_entry)) == '\0') {
+		alertpanel_error(_("SMTP server is not entered."));
+		return -1;
+	}
+	if ((protocol == A_POP3 || protocol == A_APOP || protocol == A_LOCAL) &&
+	    *gtk_entry_get_text(GTK_ENTRY(basic.uid_entry)) == '\0') {
+		alertpanel_error(_("User ID is not entered."));
+		return -1;
+	}
+	if ((protocol == A_POP3 || protocol == A_APOP) &&
+	    *gtk_entry_get_text(GTK_ENTRY(basic.recvserv_entry)) == '\0') {
+		alertpanel_error(_("POP3 server is not entered."));
+		return -1;
+	}
+	if (protocol == A_IMAP4 &&
+	    *gtk_entry_get_text(GTK_ENTRY(basic.recvserv_entry)) == '\0') {
+		alertpanel_error(_("IMAP4 server is not entered."));
+		return -1;
+	}
+	if (protocol == A_NNTP &&
+	    *gtk_entry_get_text(GTK_ENTRY(basic.nntpserv_entry)) == '\0') {
+		alertpanel_error(_("NNTP server is not entered."));
 		return -1;
 	}
 
@@ -840,9 +1027,9 @@ static void prefs_account_protocol_set_optmenu(PrefParam *pparam)
 	RecvProtocol protocol;
 	GtkOptionMenu *optmenu = GTK_OPTION_MENU(*pparam->widget);
 	GtkWidget *menu;
+	GtkWidget *menuitem;
 
 	protocol = *((RecvProtocol *)pparam->data);
-	menu = gtk_option_menu_get_menu(optmenu);
 
 	switch (protocol) {
 	case A_POP3:
@@ -854,6 +1041,100 @@ static void prefs_account_protocol_set_optmenu(PrefParam *pparam)
 	case A_IMAP4:
 		gtk_option_menu_set_history(optmenu, 2);
 		break;
+	case A_NNTP:
+		gtk_option_menu_set_history(optmenu, 3);
+		break;
+	case A_LOCAL:
+		gtk_option_menu_set_history(optmenu, 4);
+		break;
 	default:
 	}
+
+	menu = gtk_option_menu_get_menu(optmenu);
+	menuitem = gtk_menu_get_active(GTK_MENU(menu));
+	gtk_menu_item_activate(GTK_MENU_ITEM(menuitem));
+}
+
+static void prefs_account_protocol_activated(GtkMenuItem *menuitem)
+{
+	RecvProtocol protocol;
+
+	protocol = GPOINTER_TO_INT
+		(gtk_object_get_user_data(GTK_OBJECT(menuitem)));
+
+	switch(protocol) {
+	case A_NNTP:
+		gtk_widget_set_sensitive(basic.inbox_label, FALSE);
+		gtk_widget_set_sensitive(basic.inbox_entry, FALSE);
+		gtk_widget_show(basic.nntpserv_label);
+		gtk_widget_show(basic.nntpserv_entry);
+		gtk_widget_hide(basic.recvserv_label);
+		gtk_widget_hide(basic.recvserv_entry);
+		gtk_widget_hide(basic.smtpserv_label);
+		gtk_widget_hide(basic.smtpserv_entry);
+		gtk_table_set_row_spacing (GTK_TABLE (basic.serv_table), 3, 0);
+		gtk_widget_set_sensitive(basic.uid_label,  FALSE);
+		gtk_widget_set_sensitive(basic.pass_label, FALSE);
+		gtk_widget_set_sensitive(basic.uid_entry,  FALSE);
+		gtk_widget_set_sensitive(basic.pass_entry, FALSE);
+		gtk_widget_set_sensitive(receive.pop3_frame, FALSE);
+		break;
+	case A_LOCAL:
+		gtk_widget_set_sensitive(basic.inbox_label, TRUE);
+		gtk_widget_set_sensitive(basic.inbox_entry, TRUE);
+		gtk_widget_hide(basic.nntpserv_label);
+		gtk_widget_hide(basic.nntpserv_entry);
+		gtk_widget_set_sensitive(basic.recvserv_label, FALSE);
+		gtk_widget_set_sensitive(basic.recvserv_entry, FALSE);
+		gtk_widget_show(basic.recvserv_label);
+		gtk_widget_show(basic.recvserv_entry);
+		gtk_widget_show(basic.smtpserv_label);
+		gtk_widget_show(basic.smtpserv_entry);
+		gtk_table_set_row_spacing
+			(GTK_TABLE (basic.serv_table), 3, VSPACING_NARROW);
+		gtk_widget_set_sensitive(basic.uid_label,  FALSE);
+		gtk_widget_set_sensitive(basic.pass_label, FALSE);
+		gtk_widget_set_sensitive(basic.uid_entry,  FALSE);
+		gtk_widget_set_sensitive(basic.pass_entry, FALSE);
+		gtk_widget_set_sensitive(receive.pop3_frame, FALSE);
+		break;
+	case A_IMAP4:
+		gtk_widget_set_sensitive(basic.inbox_label, TRUE);
+		gtk_widget_set_sensitive(basic.inbox_entry, TRUE);
+		gtk_widget_hide(basic.nntpserv_label);
+		gtk_widget_hide(basic.nntpserv_entry);
+		gtk_widget_set_sensitive(basic.recvserv_label, TRUE);
+		gtk_widget_set_sensitive(basic.recvserv_entry, TRUE);
+		gtk_widget_show(basic.recvserv_label);
+		gtk_widget_show(basic.recvserv_entry);
+		gtk_widget_show(basic.smtpserv_label);
+		gtk_widget_show(basic.smtpserv_entry);
+		gtk_table_set_row_spacing
+			(GTK_TABLE (basic.serv_table), 3, VSPACING_NARROW);
+		gtk_widget_set_sensitive(basic.uid_label,  TRUE);
+		gtk_widget_set_sensitive(basic.pass_label, TRUE);
+		gtk_widget_set_sensitive(basic.uid_entry,  TRUE);
+		gtk_widget_set_sensitive(basic.pass_entry, TRUE);
+		gtk_widget_set_sensitive(receive.pop3_frame, FALSE);
+		break;
+	default:
+		gtk_widget_set_sensitive(basic.inbox_label, TRUE);
+		gtk_widget_set_sensitive(basic.inbox_entry, TRUE);
+		gtk_widget_hide(basic.nntpserv_label);
+		gtk_widget_hide(basic.nntpserv_entry);
+		gtk_widget_set_sensitive(basic.recvserv_label, TRUE);
+		gtk_widget_set_sensitive(basic.recvserv_entry, TRUE);
+		gtk_widget_show(basic.recvserv_label);
+		gtk_widget_show(basic.recvserv_entry);
+		gtk_widget_show(basic.smtpserv_label);
+		gtk_widget_show(basic.smtpserv_entry);
+		gtk_table_set_row_spacing
+			(GTK_TABLE (basic.serv_table), 3, VSPACING_NARROW);
+		gtk_widget_set_sensitive(basic.uid_label,  TRUE);
+		gtk_widget_set_sensitive(basic.pass_label, TRUE);
+		gtk_widget_set_sensitive(basic.uid_entry,  TRUE);
+		gtk_widget_set_sensitive(basic.pass_entry, TRUE);
+		gtk_widget_set_sensitive(receive.pop3_frame, TRUE);
+		break;
+	}
 }
