@@ -1,4 +1,21 @@
-/* procheader.c */
+/*
+ * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
+ * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
@@ -7,23 +24,17 @@
 #include <glib.h>
 #include <stdio.h>
 #include <string.h>
+#include <stdlib.h>
 #include <time.h>
 
 #include "intl.h"
 #include "procheader.h"
 #include "procmsg.h"
-#include "unmime.h"
+#include "codeconv.h"
 #include "utils.h"
 
 #define BUFFSIZE	8192
 
-#define CONV_HEADER(dest, src) \
-{ \
-	UnMimeHeader(src); \
-	conv_jistoeuc(dest, sizeof(dest), src); \
-	conv_unreadable(dest); \
-}
-
 gint procheader_get_one_field(gchar *buf, gint len, FILE *fp,
 			      HeaderEntry hentry[])
 {
@@ -34,9 +45,12 @@ gint procheader_get_one_field(gchar *buf, gint len, FILE *fp,
 	if (hentry != NULL) {
 		/* skip non-required headers */
 		do {
-			if (fgets(buf, len, fp) == NULL) return -1;
-			if (buf[0] == '\r' || buf[0] == '\n') return -1;
-			if (buf[0] == ' ' || buf[0] == '\t') continue;
+			do {
+				if (fgets(buf, len, fp) == NULL)
+					return -1;
+				if (buf[0] == '\r' || buf[0] == '\n')
+					return -1;
+			} while (buf[0] == ' ' || buf[0] == '\t');
 
 			for (hp = hentry, hnum = 0; hp->name != NULL;
 			     hp++, hnum++) {
@@ -179,21 +193,19 @@ GSList *procheader_get_header_list(const gchar *file)
 	Header *header;
 
 	if ((fp = fopen(file, "r")) == NULL) {
-		fprintf(stderr, "%s: ", file);
-		perror("fopen");
+		FILE_OP_ERROR(file, "fopen");
 		return NULL;
 	}
 
 	while (procheader_get_unfolded_line(buf, sizeof(buf), fp) != NULL) {
 		if (*buf == ':') continue;
 		for (p = buf; *p && *p != ' '; p++) {
-			if (*p == ':' && *(p + 1) == ' ') {
+			if (*p == ':') {
 				header = g_new(Header, 1);
 				header->name = g_strndup(buf, p - buf);
-				p += 2;
-				UnMimeHeader(p);
-				conv_jistoeuc(tmp, sizeof(tmp), p);
-				conv_unreadable(tmp);
+				p++;
+				while (*p == ' ' || *p == '\t') p++;
+				conv_unmime_header(tmp, sizeof(tmp), p, NULL);
 				header->body = g_strdup(tmp);
 
 				hlist = g_slist_append(hlist, header);
@@ -225,6 +237,7 @@ void procheader_get_header_fields(FILE *fp, HeaderEntry hentry[])
 	gchar buf[BUFFSIZE];
 	HeaderEntry *hp;
 	gint hnum;
+	gchar *p;
 
 	if (hentry == NULL) return;
 
@@ -232,13 +245,16 @@ void procheader_get_header_fields(FILE *fp, HeaderEntry hentry[])
 	       != -1) {
 		hp = hentry + hnum;
 
+		p = buf + strlen(hp->name);
+		while (*p == ' ' || *p == '\t') p++;
+
 		if (hp->body == NULL)
-			hp->body = g_strdup(buf + strlen(hp->name));
-		else if (!strcasecmp(hp->name, "To: ") ||
-			 !strcasecmp(hp->name, "Cc: ")) {
-			gchar *p = hp->body;
-			hp->body = g_strconcat(p, ", ", buf + 4, NULL);
-			g_free(p);
+			hp->body = g_strdup(p);
+		else if (!strcasecmp(hp->name, "To:") ||
+			 !strcasecmp(hp->name, "Cc:")) {
+			gchar *tp = hp->body;
+			hp->body = g_strconcat(tp, ", ", p, NULL);
+			g_free(tp);
 		}
 	}
 }
@@ -253,53 +269,82 @@ enum
 	H_MSG_ID	= 5,
 	H_REFERENCES	= 6,
 	H_IN_REPLY_TO	= 7,
-	H_SEEN		= 8
+	H_CONTENT_TYPE	= 8,
+	H_SEEN		= 9,
+	H_X_FACE	= 10,
 };
 
-void procheader_parse(MsgInfo *msginfo, const gchar *file)
+MsgInfo *procheader_parse(const gchar *file, MsgFlags flags, gboolean full)
 {
-	static HeaderEntry hentry[] = {{"Date: ",		NULL, FALSE},
-				       {"From: ",		NULL, TRUE},
-				       {"To: ",			NULL, TRUE},
-				       {"Newsgroups: ",		NULL, TRUE},
-				       {"Subject: ",		NULL, TRUE},
-				       {"Message-Id",		NULL, FALSE},
-				       {"References: ",		NULL, FALSE},
-				       {"In-Reply-To: ",	NULL, FALSE},
-				       {"Seen: ",		NULL, FALSE},
-				       {NULL,			NULL, FALSE}};
+	static HeaderEntry hentry_full[] = {{"Date:",		NULL, FALSE},
+					   {"From:",		NULL, TRUE},
+					   {"To:",		NULL, TRUE},
+					   {"Newsgroups:",	NULL, TRUE},
+					   {"Subject:",		NULL, TRUE},
+					   {"Message-Id:",	NULL, FALSE},
+					   {"References:",	NULL, FALSE},
+					   {"In-Reply-To:",	NULL, FALSE},
+					   {"Content-Type:",	NULL, FALSE},
+					   {"Seen:",		NULL, FALSE},
+					   {"X-Face:",		NULL, FALSE},
+					   {NULL,		NULL, FALSE}};
+
+	static HeaderEntry hentry_short[] = {{"Date:",		NULL, FALSE},
+					    {"From:",		NULL, TRUE},
+					    {"To:",		NULL, TRUE},
+					    {"Newsgroups:",	NULL, TRUE},
+					    {"Subject:",	NULL, TRUE},
+					    {"Message-Id:",	NULL, FALSE},
+					    {"References:",	NULL, FALSE},
+					    {"In-Reply-To:",	NULL, FALSE},
+					    {"Content-Type:",	NULL, FALSE},
+					    {"Seen:",		NULL, FALSE},
+					    {NULL,		NULL, FALSE}};
 
 	FILE *fp;
+	MsgInfo *msginfo;
 	gchar buf[BUFFSIZE], tmp[BUFFSIZE];
 	gchar *reference = NULL;
 	gchar *p;
+	gchar *hp;
+	HeaderEntry *hentry;
 	gint hnum;
 
+	hentry = full ? hentry_full : hentry_short;
+
 	if ((fp = fopen(file, "r")) == NULL) {
-		fprintf(stderr, "%s: ", file);
-		perror("fopen");
-		return;
+		FILE_OP_ERROR(file, "fopen");
+		return NULL;
+	}
+	if (MSG_IS_QUEUED(flags)) {
+		while (fgets(buf, sizeof(buf), fp) != NULL)
+			if (buf[0] == '\r' || buf[0] == '\n') break;
 	}
 
+	msginfo = g_new0(MsgInfo, 1);
+	msginfo->flags = flags != 0 ? flags : MSG_NEW|MSG_UNREAD;
 	msginfo->inreplyto = NULL;
 
 	while ((hnum = procheader_get_one_field(buf, sizeof(buf), fp, hentry))
 	       != -1) {
+		hp = buf + strlen(hentry[hnum].name);
+		while (*hp == ' ' || *hp == '\t') hp++;
+
 		switch (hnum) {
 		case H_DATE:
 			if (msginfo->date) break;
 			msginfo->date_t =
-				procheader_date_parse(NULL, buf + 6, 0);
-			msginfo->date = g_strdup(buf + 6);
+				procheader_date_parse(NULL, hp, 0);
+			msginfo->date = g_strdup(hp);
 			break;
 		case H_FROM:
 			if (msginfo->from) break;
-			CONV_HEADER(tmp, buf + 6);
+			conv_unmime_header(tmp, sizeof(tmp), hp, NULL);
 			msginfo->from = g_strdup(tmp);
 			msginfo->fromname = procheader_get_fromname(tmp);
 			break;
 		case H_TO:
-			CONV_HEADER(tmp, buf + 4);
+			conv_unmime_header(tmp, sizeof(tmp), hp, NULL);
 			if (msginfo->to) {
 				p = msginfo->to;
 				msginfo->to =
@@ -312,30 +357,28 @@ void procheader_parse(MsgInfo *msginfo, const gchar *file)
 			if (msginfo->newsgroups) {
 				p = msginfo->newsgroups;
 				msginfo->newsgroups =
-					g_strconcat(p, ",", buf + 12, NULL);
+					g_strconcat(p, ",", hp, NULL);
 				g_free(p);
 			} else
 				msginfo->newsgroups = g_strdup(buf + 12);
 			break;
 		case H_SUBJECT:
 			if (msginfo->subject) break;
-			CONV_HEADER(tmp, buf + 9);
+			conv_unmime_header(tmp, sizeof(tmp), hp, NULL);
 			msginfo->subject = g_strdup(tmp);
 			break;
 		case H_MSG_ID:
 			if (msginfo->msgid) break;
-			p = buf + 12;
 
-			extract_parenthesis(p, '<', '>');
-			remove_space(p);
-			msginfo->msgid = g_strdup(p);
+			extract_parenthesis(hp, '<', '>');
+			remove_space(hp);
+			msginfo->msgid = g_strdup(hp);
 			break;
 		case H_REFERENCES:
 			if (!reference) {
-				p = buf + 12;
-
-				eliminate_parenthesis(p, '(', ')');
-				if ((p = strrchr(p, '<')) != NULL) {
+				eliminate_parenthesis(hp, '(', ')');
+				if ((p = strrchr(hp, '<')) != NULL &&
+				    strchr(p + 1, '>') != NULL) {
 					extract_parenthesis(p, '<', '>');
 					remove_space(p);
 					if (*p != '\0')
@@ -345,32 +388,41 @@ void procheader_parse(MsgInfo *msginfo, const gchar *file)
 			break;
 		case H_IN_REPLY_TO:
 			if (!reference) {
-				p = buf + 13;
-
-				eliminate_parenthesis(p, '(', ')');
-				extract_parenthesis(p, '<', '>');
-				remove_space(p);
-				if (*p != '\0')
-					reference = g_strdup(buf + 13);
+				eliminate_parenthesis(hp, '(', ')');
+				extract_parenthesis(hp, '<', '>');
+				remove_space(hp);
+				if (*hp != '\0')
+					reference = g_strdup(hp);
 			}
 			break;
+		case H_CONTENT_TYPE:
+			if (!strncasecmp(hp, "multipart", 9))
+				msginfo->flags |= MSG_MIME;
+			break;
 		case H_SEEN:
-		/* mnews Seen header */
+			/* mnews Seen header */
 			MSG_UNSET_FLAGS(msginfo->flags, MSG_NEW|MSG_UNREAD);
 			break;
+		case H_X_FACE:
+			if (msginfo->xface) break;
+			msginfo->xface = g_strdup(hp);
+			break;
 		default:
 		}
 	}
 	msginfo->inreplyto = reference;
 
 	fclose(fp);
+
+	return msginfo;
 }
 
 gchar *procheader_get_fromname(const gchar *str)
 {
 	gchar *tmp, *name;
 
-	tmp = g_strdup(str);
+	Xalloca(tmp, strlen(str) + 1, return NULL);
+	strcpy(tmp, str);
 
 	if (*tmp == '\"') {
 		extract_quote(tmp, '\"');
@@ -393,7 +445,6 @@ gchar *procheader_get_fromname(const gchar *str)
 	else
 		name = g_strdup(tmp);
 
-	g_free(tmp);
 	return name;
 }
 
@@ -428,10 +479,8 @@ time_t procheader_date_parse(gchar *dest, const gchar *src, gint len)
 						zone);
 				if (result != 6) {
 					g_warning("Invalid date: %s\n", src);
-					if (dest && len > 0) {
-						strncpy(dest, src, len - 1);
-						dest[len - 1] = '\0';
-					}
+					if (dest && len > 0)
+						strncpy2(dest, src, len);
 					return 0;
 				}
 			}
@@ -449,7 +498,7 @@ time_t procheader_date_parse(gchar *dest, const gchar *src, gint len)
 	if ((p = strstr(monthstr, month)) != NULL)
 		dmonth = (gint)(p - monthstr) / 3 + 1;
 	else {
-		g_print(_("Invalid month\n"));
+		g_warning("Invalid month: %s\n", month);
 		dmonth = G_DATE_BAD_MONTH;
 	}
 
@@ -485,13 +534,12 @@ void procheader_date_get_localtime(gchar *dest, gint len, const time_t timer)
 	lt = localtime(&timer);
 
 	wdlen = strlen(tr_wday) / 7;
-	wday = g_malloc(wdlen + 1);
-	strncpy(wday, tr_wday + lt->tm_wday * wdlen, wdlen);
-	wday[wdlen] = '\0';
+	if ((wday = alloca(wdlen + 1))) {
+		strncpy(wday, tr_wday + lt->tm_wday * wdlen, wdlen);
+		wday[wdlen] = '\0';
+	}
 
-	g_snprintf(dest, len, "%d/%d/%d(%s) %02d:%02d",
-		   lt->tm_year + 1900, lt->tm_mon + 1, lt->tm_mday,
+	g_snprintf(dest, len, "%02d/%d/%d(%s) %02d:%02d",
+		   lt->tm_year % 100, lt->tm_mon + 1, lt->tm_mday,
 		   wday, lt->tm_hour, lt->tm_min);
-
-	g_free(wday);
 }
