@@ -1,4 +1,21 @@
-/* smtp.c */
+/*
+ * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
+ * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
 
 #include <glib.h>
 #include <stdio.h>
@@ -27,9 +44,9 @@ gint smtp_from(gint sock, const gchar *from)
 	gchar buf[MSGBUFSIZE];
 
 	if (strchr(from, '<'))
-		snprintf(buf, sizeof(buf), "MAIL FROM: %s", from);
+		g_snprintf(buf, sizeof(buf), "MAIL FROM: %s", from);
 	else
-		snprintf(buf, sizeof(buf), "MAIL FROM: <%s>", from);
+		g_snprintf(buf, sizeof(buf), "MAIL FROM: <%s>", from);
 
 	sock_printf(sock, "%s\r\n", buf);
 	if (verbose)
@@ -43,9 +60,9 @@ gint smtp_rcpt(gint sock, const gchar *to)
 	gchar buf[MSGBUFSIZE];
 
 	if (strchr(to, '<'))
-		snprintf(buf, sizeof(buf), "RCPT TO: %s", to);
+		g_snprintf(buf, sizeof(buf), "RCPT TO: %s", to);
 	else
-		snprintf(buf, sizeof(buf), "RCPT TO: <%s>", to);
+		g_snprintf(buf, sizeof(buf), "RCPT TO: <%s>", to);
 
 	sock_printf(sock, "%s\r\n", buf);
 	if (verbose)
