@@ -1,175 +1,96 @@
-/* utils.c */
+/*
+ * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
+ * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ *
+ * This program is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ */
 
 #ifdef HAVE_CONFIG_H
 #  include "config.h"
 #endif
 
-#undef USE_ICONV
+#include "defs.h"
 
 #include <glib.h>
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
 #include <errno.h>
-#ifdef USE_ICONV
-#  include <iconv.h>
-#endif /* USE_ICONV */
 
-#include <wchar.h>
-#include <wctype.h>
+#if (HAVE_WCTYPE_H && HAVE_WCHAR_H)
+#  include <wchar.h>
+#  include <wctype.h>
+#endif
 #include <stdlib.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <stdarg.h>
-#include <kcc.h>
 #include <sys/types.h>
+#include <sys/wait.h>
 #include <dirent.h>
+#include <time.h>
 
 #include "intl.h"
-#include "main.h"
 #include "utils.h"
 #include "statusbar.h"
 #include "logwindow.h"
 
 #define BUFFSIZE	8192
 
-#define iskanji(c) \
-	(((c) & 0xff) >= 0xa1 && ((c) & 0xff) <= 0xfe)
-#define iseucss(c) \
-	(((c) & 0xff) == 0x8e || ((c) & 0xff) == 0x8f)
-#define isunprintablekanji(c) \
-	(((c) & 0xff) >= 0xa9 && ((c) & 0xff) <= 0xaf)
-
 extern gboolean debug_mode;
 
-void list_remove_all(GList *list)
+static void hash_free_strings_func(gpointer key, gpointer value, gpointer data);
+
+void list_free_strings(GList *list)
 {
-	while (list != NULL)
-		list = g_list_remove(list, list->data);
+	list = g_list_first(list);
+
+	while (list != NULL) {
+		g_free(list->data);
+		list = list->next;
+	}
 }
 
-void slist_remove_all(GSList *list)
+void slist_free_strings(GSList *list)
 {
-	while (list != NULL)
-		list = g_slist_remove(list, list->data);
+	while (list != NULL) {
+		g_free(list->data);
+		list = list->next;
+	}
 }
 
-void conv_jistoeuc(gchar *outbuf, gint outlen, const gchar *inbuf)
+static void hash_free_strings_func(gpointer key, gpointer value, gpointer data)
 {
-#ifdef USE_ICONV
-	static iconv_t cd;
-	size_t inlen;
-
-	if (!cd)
-		cd = iconv_open("EUC-JP", OUT_CHARSET);
-
-	inlen = strlen(inbuf) + 1;
-	memset(outbuf, 0, inlen);
-	iconv(cd, &inbuf, &inlen, &outbuf, &outlen);
-#else
-	KCC_filter(outbuf, "EUC", (gchar *)inbuf, "JISBB", 0, 0, 0);
-#endif /* USE_ICONV */
+	g_free(key);
 }
 
-void conv_euctojis(gchar *outbuf, gint outlen, const gchar *inbuf)
+void hash_free_strings(GHashTable *table)
 {
-#ifdef USE_ICONV
-	static iconv_t cd;
-	size_t inlen;
-
-	if (!cd)
-		cd = iconv_open(OUT_CHARSET, "EUC-JP");
-
-	inlen = strlen(inbuf) + 1;
-	memset(outbuf, 0, MIN(inlen * 3, outlen));
-	iconv(cd, &inbuf, &inlen, &outbuf, &outlen);
-#else
-	size_t inlen, len;
-
-	inlen = strlen(inbuf);
-	if (iskanji(inbuf[inlen - 1]) || iseucss(inbuf[inlen - 1])) {
-		/* if tail end of the string is not ended with ascii,
-		   add dummy return code. */
-		gchar *tmpin, *tmpout;
-
-		/* length of original string + '\n' + '\0' */
-		tmpin = g_malloc(inlen + 2);
-		if (tmpin == NULL) {
-			g_warning(_("can't allocate memory\n"));
-			KCC_filter(outbuf, "JISBB", (gchar *)inbuf, "EUC",
-				   0, 0, 0);
-			return;
-		}
-		strcpy(tmpin, inbuf);
-		tmpin[inlen] = '\n';
-		tmpin[inlen + 1] = '\0';
-
-		tmpout = g_malloc(outlen + 1);
-		if (tmpout == NULL) {
-			g_warning(_("can't allocate memory\n"));
-			g_free(tmpin);
-			KCC_filter(outbuf, "JISBB", (gchar *)inbuf, "EUC",
-				   0, 0, 0);
-			return;
-		}
-
-		KCC_filter(tmpout, "JISBB", tmpin, "EUC", 0, 0, 0);
-		len = strlen(tmpout);
-		if (tmpout[len - 1] == '\n')
-			tmpout[len - 1] = '\0';
-		strncpy(outbuf, tmpout, outlen);
-		outbuf[outlen - 1] = '\0';
-
-		g_free(tmpin);
-		g_free(tmpout);
-	} else
-		KCC_filter(outbuf, "JISBB", (gchar *)inbuf, "EUC", 0, 0, 0);
-#endif /* USE_ICONV */
+	g_hash_table_foreach(table, hash_free_strings_func, NULL);
 }
 
-void conv_anytoeuc(gchar *outbuf, gint outlen, const gchar *inbuf)
+void ptr_array_free_strings(GPtrArray *array)
 {
-	KCC_filter(outbuf, "EUC", (gchar *)inbuf, "AUTO", 0, 0, 0);
-}
+	gint i;
+	gchar *str;
 
-void conv_anytojis(gchar *outbuf, gint outlen, const gchar *inbuf)
-{
-	KCC_filter(outbuf, "JISBB", (gchar *)inbuf, "AUTO", 0, 0, 0);
-}
+	g_return_if_fail(array != NULL);
 
-#define SUBST_CHAR	'_'
-
-void conv_unreadable(gchar *str)
-{
-	register gchar *p = str;
-
-	while (*p != '\0') {
-		if (isascii(*p))
-			/* printable 7 bit code */
-			p++;
-		else if (iskanji(*p)) {
-			if (iskanji(*(p + 1)) && !isunprintablekanji(*p))
-				/* printable euc-jp code */
-				p += 2;
-			else {
-				/* substitute unprintable code */
-				*p++ = SUBST_CHAR;
-				if (*p != '\0') {
-					if (isascii(*p))
-						p++;
-					else
-						*p++ = SUBST_CHAR;
-				}
-			}
-		} else if (iseucss(*p)) {
-			if ((*(p + 1) & 0x80) != 0)
-				/* euc-jp hankaku kana */
-				p += 2;
-			else
-				*p++ = SUBST_CHAR;
-		} else
-			/* substitute unprintable 1 byte code */
-			*p++ = SUBST_CHAR;
+	for (i = 0; i < array->len; i++) {
+		str = g_ptr_array_index(array, i);
+		g_free(str);
 	}
 }
 
@@ -241,8 +162,8 @@ gint path_cmp(const gchar *s1, const gchar *s2)
 	len1 = strlen(s1);
 	len2 = strlen(s2);
 
-	if (s1[len1 - 1] == G_DIR_SEPARATOR) len1 -= 1;
-	if (s2[len2 - 1] == G_DIR_SEPARATOR) len2 -= 1;
+	if (s1[len1 - 1] == G_DIR_SEPARATOR) len1--;
+	if (s2[len2 - 1] == G_DIR_SEPARATOR) len2--;
 
 	return strncmp(s1, s2, MAX(len1, len2));
 }
@@ -285,6 +206,100 @@ gchar *strcasestr(const gchar *haystack, const gchar *needle)
 	return NULL;
 }
 
+/* Copy no more than N characters of SRC to DEST, with NULL terminating.  */
+gchar *strncpy2(gchar *dest, const gchar *src, size_t n)
+{
+	register gchar c;
+	gchar *s = dest;
+
+	do {
+		if (--n == 0) {
+			*dest = '\0';
+			return s;
+		}
+		c = *src++;
+		*dest++ = c;
+	} while (c != '\0');
+
+	/* don't do zero fill */
+	return s;
+}
+
+#if !HAVE_ISWALNUM
+int iswalnum(wint_t wc)
+{
+	return isalnum((int)wc);
+}
+#endif
+
+#if !HAVE_ISWSPACE
+int iswspace(wint_t wc)
+{
+	return isspace((int)wc);
+}
+#endif
+
+#if !HAVE_TOWLOWER
+wint_t towlower(wint_t wc)
+{
+	if (wc >= L'A' && wc <= L'Z')
+		return wc + L'a' - L'A';
+
+	return wc;
+}
+#endif
+
+#if !HAVE_WCSLEN
+size_t wcslen(const wchar_t *s)
+{
+	size_t len = 0;
+
+	while (*s != L'\0')
+		++len, ++s;
+
+	return len;
+}
+#endif
+
+#if !HAVE_WCSCPY
+/* Copy SRC to DEST.  */
+wchar_t *wcscpy(wchar_t *dest, const wchar_t *src)
+{
+	wint_t c;
+	wchar_t *s = dest;
+
+	do {
+		c = *src++;
+		*dest++ = c;
+	} while (c != L'\0');
+
+	return s;
+}
+#endif
+
+#if !HAVE_WCSNCPY
+/* Copy no more than N wide-characters of SRC to DEST.  */
+wchar_t *wcsncpy (wchar_t *dest, const wchar_t *src, size_t n)
+{
+	wint_t c;
+	wchar_t *s = dest;
+
+	do {
+		c = *src++;
+		*dest++ = c;
+		if (--n == 0)
+			return s;
+	} while (c != L'\0');
+
+	/* zero fill */
+	do
+		*dest++ = L'\0';
+	while (--n > 0);
+
+	return s;
+}
+#endif
+
 /* Duplicate S, returning an identical malloc'd string. */
 wchar_t *wcsdup(const wchar_t *s)
 {
@@ -321,7 +336,12 @@ wchar_t *strdup_mbstowcs(const gchar *s)
 
 	if (s) {
 		new_str = g_new(wchar_t, strlen(s) + 1);
-		mbstowcs(new_str, s, strlen(s) + 1);
+		if (mbstowcs(new_str, s, strlen(s) + 1) < 0) {
+			g_free(new_str);
+			new_str = NULL;
+		} else
+			new_str = g_realloc(new_str,
+					    sizeof(wchar_t) * (wcslen(new_str) + 1));
 	} else
 		new_str = NULL;
 
@@ -331,11 +351,16 @@ wchar_t *strdup_mbstowcs(const gchar *s)
 gchar *strdup_wcstombs(const wchar_t *s)
 {
 	gchar *new_str;
+	size_t len;
 
 	if (s) {
-		new_str = g_new(gchar, wcslen(s) * MB_CUR_MAX + 1);
-		wcstombs(new_str, s, wcslen(s) * MB_CUR_MAX + 1);
-		//new_str = g_realloc(new_str, strlen(new_str) + 1);
+		len = wcslen(s) * MB_CUR_MAX + 1;
+		new_str = g_new(gchar, len);
+		if (wcstombs(new_str, s, len) < 0) {
+			g_free(new_str);
+			new_str = NULL;
+		} else
+			new_str = g_realloc(new_str, strlen(new_str) + 1);
 	} else
 		new_str = NULL;
 
@@ -388,7 +413,7 @@ gboolean is_next_mbs(const wchar_t *s)
 {
 	gint mbl;
 	const wchar_t *wp;
-	gchar tmp[sizeof(wchar_t)];
+	gchar tmp[MB_CUR_MAX];
 
 	/* skip head space */
 	for (wp = s; *wp != (wchar_t)0 && iswspace(*wp); wp++)
@@ -425,17 +450,15 @@ gint subject_compare(const gchar *s1, const gchar *s2)
 	if (!s1 || !s2) return -1;
 	if (!*s1 || !*s2) return -1;
 
-	str1 = g_strdup(s1);
-	str2 = g_strdup(s2);
+	Xalloca(str1, strlen(s1) + 1, return -1);
+	Xalloca(str2, strlen(s2) + 1, return -1);
+	strcpy(str1, s1);
+	strcpy(str2, s2);
 
 	trim_subject(str1);
 	trim_subject(str2);
 
-	if (!*str1 || !*str2) {
-		g_free(str1);
-		g_free(str2);
-		return -1;
-	}
+	if (!*str1 || !*str2) return -1;
 
 	retval = strcmp(str1, str2);
 	//if (retval == 0)
@@ -443,9 +466,6 @@ gint subject_compare(const gchar *s1, const gchar *s2)
 	//		"str1 = %s\nstr2 = %s\nmatched.\n",
 	//		s1, s2, str1, str2);
 
-	g_free(str1);
-	g_free(str2);
-
 	return retval;
 }
 
@@ -515,6 +535,37 @@ void extract_parenthesis(gchar *str, gchar op, gchar cl)
 	*destp = '\0';
 }
 
+void extract_parenthesis_with_skip_quote(gchar *str, gchar quote_chr,
+					 gchar op, gchar cl)
+{
+	register gchar *srcp, *destp;
+	gint in_brace;
+	gboolean in_quote = FALSE;
+
+	srcp = destp = str;
+
+	while ((srcp = strchr_with_skip_quote(destp, '"', op))) {
+		if (destp > str)
+			*destp++ = ' ';
+		memmove(destp, srcp + 1, strlen(srcp));
+		in_brace = 1;
+		while(*destp) {
+			if (*destp == op && !in_quote)
+				in_brace++;
+			else if (*destp == cl && !in_quote)
+				in_brace--;
+			else if (*destp == quote_chr)
+				in_quote ^= TRUE;
+
+			if (in_brace == 0)
+				break;
+
+			destp++;
+		}
+	}
+	*destp = '\0';
+}
+
 void eliminate_quote(gchar *str, gchar quote_chr)
 {
 	register gchar *srcp, *destp;
@@ -545,6 +596,153 @@ void extract_quote(gchar *str, gchar quote_chr)
 	}
 }
 
+void eliminate_address_comment(gchar *str)
+{
+	register gchar *srcp, *destp;
+	gint in_brace;
+
+	srcp = destp = str;
+
+	while ((destp = strchr(destp, '"'))) {
+		if ((srcp = strchr(destp + 1, '"'))) {
+			srcp++;
+			if (*srcp == '@') {
+				destp = srcp + 1;
+			} else {
+				while (isspace(*srcp)) srcp++;
+				memmove(destp, srcp, strlen(srcp) + 1);
+			}
+		} else {
+			*destp = '\0';
+			break;
+		}
+	}
+
+	srcp = destp = str;
+
+	while ((destp = strchr_with_skip_quote(destp, '"', '('))) {
+		in_brace = 1;
+		srcp = destp + 1;
+		while (*srcp) {
+			if (*srcp == '(')
+				in_brace++;
+			else if (*srcp == ')')
+				in_brace--;
+			srcp++;
+			if (in_brace == 0)
+				break;
+		}
+		while (isspace(*srcp)) srcp++;
+		memmove(destp, srcp, strlen(srcp) + 1);
+	}
+}
+
+gchar *strchr_with_skip_quote(const gchar *str, gint quote_chr, gint c)
+{
+	gboolean in_quote = FALSE;
+
+	while (*str) {
+		if (*str == c && !in_quote)
+			return (gchar *)str;
+		if (*str == quote_chr)
+			in_quote ^= TRUE;
+		str++;
+	}
+
+	return NULL;
+}
+
+gchar *strrchr_with_skip_quote(const gchar *str, gint quote_chr, gint c)
+{
+	gboolean in_quote = FALSE;
+	const gchar *p;
+
+	p = str + strlen(str) - 1;
+	while (p >= str) {
+		if (*p == c && !in_quote)
+			return (gchar *)p;
+		if (*p == quote_chr)
+			in_quote ^= TRUE;
+		p--;
+	}
+
+	return NULL;
+}
+
+void extract_address(gchar *str)
+{
+	eliminate_address_comment(str);
+	if (strchr_with_skip_quote(str, '"', '<'))
+		extract_parenthesis_with_skip_quote(str, '"', '<', '>');
+	g_strstrip(str);
+}
+
+GSList *address_list_append(GSList *addr_list, const gchar *str)
+{
+	gchar *work;
+	gchar *workp;
+
+	if (!str) return addr_list;
+
+	Xstrdup_a(work, str, return addr_list);
+
+	eliminate_address_comment(work);
+	workp = work;
+
+	while (workp && *workp) {
+		gchar *p, *next;
+
+		if ((p = strchr_with_skip_quote(workp, '"', ','))) {
+			*p = '\0';
+			next = p + 1;
+		} else
+			next = NULL;
+
+		if (strchr_with_skip_quote(workp, '"', '<'))
+			extract_parenthesis_with_skip_quote
+				(workp, '"', '<', '>');
+
+		g_strstrip(workp);
+		if (*workp)
+			addr_list = g_slist_append(addr_list, g_strdup(workp));
+
+		workp = next;
+	}
+
+	return addr_list;
+}
+
+GSList *newsgroup_list_append(GSList *group_list, const gchar *str)
+{
+	gchar *work;
+	gchar *workp;
+
+	if (!str) return group_list;
+
+	Xstrdup_a(work, str, return group_list);
+
+	workp = work;
+
+	while (workp && *workp) {
+		gchar *p, *next;
+
+		if ((p = strchr_with_skip_quote(workp, '"', ','))) {
+			*p = '\0';
+			next = p + 1;
+		} else
+			next = NULL;
+
+		g_strstrip(workp);
+		if (*workp)
+			group_list = g_slist_append(group_list,
+						    g_strdup(workp));
+
+		workp = next;
+	}
+
+	return group_list;
+}
+
 void remove_return(gchar *str)
 {
 	register gchar *p = str;
@@ -552,7 +750,8 @@ void remove_return(gchar *str)
 	while (*p) {
 		if (*p == '\n' || *p == '\r')
 			memmove(p, p + 1, strlen(p));
-		p++;
+		else
+			p++;
 	}
 }
 
@@ -567,7 +766,8 @@ void remove_space(gchar *str)
 			spc++;
 		if (spc)
 			memmove(p, p + spc, strlen(p + spc) + 1);
-		p++;
+		else
+			p++;
 	}
 }
 
@@ -584,10 +784,10 @@ void subst_char(gchar *str, gchar orig, gchar subst)
 
 gboolean is_header_line(const gchar *str)
 {
-	if (*str == ':') return FALSE;
+	if (str[0] == ':') return FALSE;
 
 	while (*str != '\0' && *str != ' ') {
-		if (*str == ':' && *(str + 1) == ' ')
+		if (*str == ':')
 			return TRUE;
 		str++;
 	}
@@ -595,6 +795,52 @@ gboolean is_header_line(const gchar *str)
 	return FALSE;
 }
 
+gboolean is_ascii_str(const guchar *str)
+{
+	while (*str != '\0') {
+		if (*str != '\t' && *str != ' ' &&
+		    *str != '\r' && *str != '\n' &&
+		    (*str < 32 || *str >= 127))
+			return FALSE;
+		str++;
+	}
+
+	return TRUE;
+}
+
+GList *uri_list_extract_filenames(const gchar *uri_list)
+{
+	GList *result = NULL;
+	const gchar *p, *q;
+	gchar *file;
+
+	p = uri_list;
+
+	while (p) {
+		if (*p != '#') {
+			while (isspace(*p)) p++;
+			if (!strncmp(p, "file:", 5)) {
+				p += 5;
+				q = p;
+				while (*q && *q != '\n' && *q != '\r') q++;
+
+				if (q > p) {
+					q--;
+					while (q > p && isspace(*q)) q--;
+					file = g_malloc(q - p + 2);
+					strncpy(file, p, q - p + 1);
+					file[q - p + 1] = '\0';
+					result = g_list_append(result,file);
+				}
+			}
+		}
+		p = strchr(p, '\n');
+		if (p) p++;
+	}
+
+	return result;
+}
+
 gchar *get_rc_dir(void)
 {
 	static gchar *rc_dir = NULL;
@@ -617,21 +863,49 @@ gchar *get_news_cache_dir(void)
 	return news_cache_dir;
 }
 
+gchar *get_imap_cache_dir(void)
+{
+	static gchar *imap_cache_dir = NULL;
+
+	if (!imap_cache_dir)
+		imap_cache_dir = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S,
+					     IMAP_CACHE_DIR, NULL);
+
+	return imap_cache_dir;
+}
+
+gchar *get_mime_tmp_dir(void)
+{
+	static gchar *mime_tmp_dir = NULL;
+
+	if (!mime_tmp_dir)
+		mime_tmp_dir = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S,
+					   MIME_TMP_DIR, NULL);
+
+	return mime_tmp_dir;
+}
+
+gchar *get_tmp_file(void)
+{
+	static gchar *tmp_file = NULL;
+
+	if (!tmp_file)
+		tmp_file = g_strconcat(get_rc_dir(), G_DIR_SEPARATOR_S,
+				       "tmpfile", NULL);
+
+	return tmp_file;
+}
+
 gchar *get_domain_name(void)
 {
 	static gchar *domain_name = NULL;
 
 	if (!domain_name) {
 		gchar buf[BUFFSIZE] = "";
-		gint val;
 
-		if ((val = getdomainname(buf, sizeof(buf))) < 0)
-			perror("getdomainname");
-		if (val < 0 || buf[0] == '(' || buf[0] == '\0') {
-			if (gethostname(buf, sizeof(buf)) < 0) {
-				perror("gethostname");
-				strcpy(buf, "unknown");
-			}
+		if (gethostname(buf, sizeof(buf)) < 0) {
+			perror("gethostname");
+			strcpy(buf, "unknown");
 		}
 
 		domain_name = g_strdup(buf);
@@ -645,8 +919,7 @@ off_t get_file_size(const gchar *file)
 	struct stat s;
 
 	if (stat(file, &s) < 0) {
-		fprintf(stderr, "%s: ", file);
-		perror("stat");
+		FILE_OP_ERROR(file, "stat");
 		return -1;
 	}
 
@@ -658,10 +931,7 @@ gboolean is_file_exist(const gchar *file)
 	struct stat s;
 
 	if (stat(file, &s) < 0) {
-		if (ENOENT != errno) {
-			fprintf(stderr, "%s: ", file);
-			perror("stat");
-		}
+		if (ENOENT != errno) FILE_OP_ERROR(file, "stat");
 		return FALSE;
 	}
 
@@ -676,8 +946,7 @@ gboolean is_dir_exist(const gchar *dir)
 	struct stat s;
 
 	if (stat(dir, &s) < 0) {
-		//fprintf(stderr, "%s: ", dir);
-		//perror("stat");
+		if (ENOENT != errno) FILE_OP_ERROR(dir, "stat");
 		return FALSE;
 	}
 
@@ -695,10 +964,8 @@ gint change_dir(const gchar *dir)
 		prevdir = g_get_current_dir();
 
 	if (chdir(dir) < 0) {
-		fprintf(stderr, "%s: ", dir);
-		perror("chdir");
-		if (debug_mode);
-			g_free(prevdir);
+		FILE_OP_ERROR(dir, "chdir");
+		if (debug_mode) g_free(prevdir);
 		return -1;
 	} else if (debug_mode) {
 		gchar *cwd;
@@ -713,6 +980,78 @@ gint change_dir(const gchar *dir)
 	return 0;
 }
 
+gint make_dir_hier(const gchar *dir)
+{
+	gchar *parent_dir;
+	const gchar *p;
+
+	for (p = dir; (p = strchr(p, G_DIR_SEPARATOR)) != NULL; p++) {
+		parent_dir = g_strndup(dir, p - dir);
+		if (*parent_dir != '\0') {
+			if (!is_dir_exist(parent_dir)) {
+				if (mkdir(parent_dir, S_IRWXU) < 0) {
+					FILE_OP_ERROR(parent_dir, "mkdir");
+					g_free(parent_dir);
+					return -1;
+				}
+				if (chmod(parent_dir, S_IRWXU) < 0)
+					FILE_OP_ERROR(parent_dir, "chmod");
+			}
+		}
+		g_free(parent_dir);
+	}
+	if (!is_dir_exist(dir)) {
+		if (mkdir(dir, S_IRWXU) < 0) {
+			FILE_OP_ERROR(dir, "mkdir");
+			return -1;
+		}
+		if (chmod(dir, S_IRWXU) < 0)
+			FILE_OP_ERROR(dir, "chmod");
+	}
+
+	return 0;
+}
+
+gint remove_all_files(const gchar *dir)
+{
+	DIR *dp;
+	struct dirent *d;
+	gchar *prev_dir;
+
+	prev_dir = g_get_current_dir();
+
+	if (chdir(dir) < 0) {
+		FILE_OP_ERROR(dir, "chdir");
+		return -1;
+	}
+
+	if ((dp = opendir(".")) == NULL) {
+		FILE_OP_ERROR(dir, "opendir");
+		return -1;
+	}
+
+	while ((d = readdir(dp)) != NULL) {
+		if (!strcmp(d->d_name, ".") ||
+		    !strcmp(d->d_name, ".."))
+			continue;
+
+		if (unlink(d->d_name) < 0)
+			FILE_OP_ERROR(d->d_name, "unlink");
+	}
+
+	closedir(dp);
+
+	if (chdir(prev_dir) < 0) {
+		FILE_OP_ERROR(prev_dir, "chdir");
+		g_free(prev_dir);
+		return -1;
+	}
+
+	g_free(prev_dir);
+
+	return 0;
+}
+
 gint remove_dir_recursive(const gchar *dir)
 {
 	struct stat s;
@@ -723,16 +1062,14 @@ gint remove_dir_recursive(const gchar *dir)
 	//g_print("dir = %s\n", dir);
 
 	if (stat(dir, &s) < 0) {
-		g_warning("can't stat %s\n", dir);
-		perror("stat");
+		FILE_OP_ERROR(dir, "stat");
 		if (ENOENT == errno) return 0;
 		return -1;
 	}
 
 	if (!S_ISDIR(s.st_mode)) {
 		if (unlink(dir) < 0) {
-			fprintf(stderr, "%s: ", dir);
-			perror("unlink");
+			FILE_OP_ERROR(dir, "unlink");
 			return -1;
 		}
 
@@ -742,15 +1079,23 @@ gint remove_dir_recursive(const gchar *dir)
 	prev_dir = g_get_current_dir();
 	//g_print("prev_dir = %s\n", prev_dir);
 
+	if (!path_cmp(prev_dir, dir)) {
+		g_free(prev_dir);
+		if (chdir("..") < 0) {
+			FILE_OP_ERROR(dir, "chdir");
+			return -1;
+		}
+		prev_dir = g_get_current_dir();
+	}
+
 	if (chdir(dir) < 0) {
-		fprintf(stderr, "%s: ", dir);
-		perror("chdir");
+		FILE_OP_ERROR(dir, "chdir");
 		g_free(prev_dir);
 		return -1;
 	}
 
 	if ((dp = opendir(".")) == NULL) {
-		perror("opendir");
+		FILE_OP_ERROR(dir, "opendir");
 		chdir(prev_dir);
 		g_free(prev_dir);
 		return -1;
@@ -763,8 +1108,7 @@ gint remove_dir_recursive(const gchar *dir)
 			continue;
 
 		if (stat(d->d_name, &s) < 0) {
-			fprintf(stderr, "%s: ", d->d_name);
-			perror("stat");
+			FILE_OP_ERROR(d->d_name, "stat");
 			continue;
 		}
 
@@ -776,18 +1120,15 @@ gint remove_dir_recursive(const gchar *dir)
 				return -1;
 			}
 		} else {
-			if (unlink(d->d_name) < 0) {
-				fprintf(stderr, "%s: ", d->d_name);
-				perror("unlink");
-			}
+			if (unlink(d->d_name) < 0)
+				FILE_OP_ERROR(d->d_name, "unlink");
 		}
 	}
 
 	closedir(dp);
 
 	if (chdir(prev_dir) < 0) {
-		fprintf(stderr, "%s: ", prev_dir);
-		perror("chdir");
+		FILE_OP_ERROR(prev_dir, "chdir");
 		g_free(prev_dir);
 		return -1;
 	}
@@ -795,8 +1136,7 @@ gint remove_dir_recursive(const gchar *dir)
 	g_free(prev_dir);
 
 	if (rmdir(dir) < 0) {
-		fprintf(stderr, "%s: ", dir);
-		perror("rmdir");
+		FILE_OP_ERROR(dir, "rmdir");
 		return -1;
 	}
 
@@ -810,37 +1150,31 @@ gint copy_file(const gchar *src, const gchar *dest)
 	gchar *dest_bak = NULL;
 
 	if ((src_fp = fopen(src, "r")) == NULL) {
-		fprintf(stderr, "%s: ", src);
-		perror("fopen");
+		FILE_OP_ERROR(src, "fopen");
 		return -1;
 	}
 	if (is_file_exist(dest)) {
 		dest_bak = g_strconcat(dest, ".bak", NULL);
 		if (rename(dest, dest_bak) < 0) {
-			fprintf(stderr, "%s: ", dest);
-			perror("rename");
+			FILE_OP_ERROR(dest, "rename");
 			fclose(src_fp);
 			g_free(dest_bak);
 			return -1;
 		}
 	}
 	if ((dest_fp = fopen(dest, "w")) == NULL) {
-		fprintf(stderr, "%s: ", dest);
-		perror("fopen");
+		FILE_OP_ERROR(dest, "fopen");
 		fclose(src_fp);
 		if (dest_bak) {
-			if (rename(dest_bak, dest) < 0) {
-				fprintf(stderr, "%s: ", dest_bak);
-				perror("rename");
-			}
+			if (rename(dest_bak, dest) < 0)
+				FILE_OP_ERROR(dest_bak, "rename");
 			g_free(dest_bak);
 		}
 		return -1;
 	}
 
 	if (chmod(dest, S_IRUSR|S_IWUSR) < 0) {
-		fprintf(stderr, "%s: ", dest);
-		perror("chmod");
+		FILE_OP_ERROR(dest, "chmod");
 		g_warning(_("can't change file mode\n"));
 	}
 
@@ -851,22 +1185,160 @@ gint copy_file(const gchar *src, const gchar *dest)
 			fclose(src_fp);
 			unlink(dest);
 			if (dest_bak) {
-				if (rename(dest_bak, dest) < 0) {
-					fprintf(stderr, "%s: ", dest_bak);
-					perror("rename");
-				}
+				if (rename(dest_bak, dest) < 0)
+					FILE_OP_ERROR(dest_bak, "rename");
 				g_free(dest_bak);
 			}
 			return -1;
 		}
 	}
 
-	fclose(dest_fp);
 	fclose(src_fp);
+	if (fclose(dest_fp) == EOF) {
+		FILE_OP_ERROR(dest, "fclose");
+		unlink(dest);
+		if (dest_bak) {
+			if (rename(dest_bak, dest) < 0)
+				FILE_OP_ERROR(dest_bak, "rename");
+			g_free(dest_bak);
+		}
+		return -1;
+	}
 	g_free(dest_bak);
+
 	return 0;
 }
 
+gint execute_async(gchar *const argv[])
+{
+	pid_t pid;
+
+	if ((pid = fork()) < 0) {
+		perror("fork");
+		return -1;
+	}
+
+	if (pid == 0) {			/* child process */
+		pid_t gch_pid;
+
+		if ((gch_pid = fork()) < 0) {
+			perror("fork");
+			_exit(1);
+		}
+
+		if (gch_pid == 0) {	/* grandchild process */
+			execvp(argv[0], argv);
+
+			perror("execvp");
+			_exit(1);
+		}
+
+		_exit(0);
+	}
+
+	waitpid(pid, NULL, 0);
+
+	return 0;
+}
+
+gint execute_command_line(const gchar *cmdline)
+{
+	gchar **argv;
+	gint i;
+	gint ret;
+
+	argv = g_strsplit(cmdline, " ", strlen(cmdline));
+
+	for (i = 0; argv[i] != NULL; i++) {
+		gchar *str = argv[i];
+
+		if (str[0] == '\'' || str[0] == '\"') {
+			gint len;
+
+			len = strlen(str);
+			if (str[len - 1] == str[0]) {
+				str[len - 1] = '\0';
+				memmove(str, str + 1, len);
+			}
+		}
+	}
+
+	ret = execute_async(argv);
+	g_strfreev(argv);
+
+	return ret;
+}
+
+gint open_uri(const gchar *uri, const gchar *cmdline)
+{
+	static gchar *default_cmdline = "netscape -remote openURL(%s,raise)";
+	gchar buf[BUFFSIZE];
+	gchar *p;
+
+	g_return_val_if_fail(uri != NULL, -1);
+
+	if (cmdline &&
+	    (p = strchr(cmdline, '%')) && *(p + 1) == 's' &&
+	    !strchr(p + 2, '%'))
+		g_snprintf(buf, sizeof(buf), cmdline, uri);
+	else {
+		if (cmdline)
+			g_warning(_("Open URI command line is invalid: `%s'"),
+				  cmdline);
+		g_snprintf(buf, sizeof(buf), default_cmdline, uri);
+	}
+
+	execute_command_line(buf);
+
+	return 0;
+}
+
+/* calculate timezone offset */
+gchar *tzoffset(time_t *now)
+{
+	static gchar offset_string[6];
+	struct tm gmt, *lt;
+	gint off;
+	gchar sign = '+';
+
+	gmt = *gmtime(now);
+	lt = localtime(now);
+	off = (lt->tm_hour - gmt.tm_hour) * 60 + lt->tm_min - gmt.tm_min;
+	if (lt->tm_year < gmt.tm_year)
+		off -= 24 * 60;
+	else if (lt->tm_year > gmt.tm_year)
+		off += 24 * 60;
+	else if (lt->tm_yday < gmt.tm_yday)
+		off -= 24 * 60;
+	else if (lt->tm_yday > gmt.tm_yday)
+		off += 24 * 60;
+	if (off < 0) {
+		sign = '-';
+		off = -off;
+	}
+	if (off >= 24 * 60)		/* should be impossible */
+		off = 23 * 60 + 59;	/* if not, insert silly value */
+	sprintf(offset_string, "%c%02d%02d", sign, off / 60, off % 60);
+
+	return (offset_string);
+}
+
+void get_rfc822_date(gchar *buf, gint len)
+{
+	struct tm *lt;
+	time_t t;
+	gchar day[4], mon[4];
+	gint dd, hh, mm, ss, yyyy;
+
+	t = time(NULL);
+	lt = localtime(&t);
+
+	sscanf(asctime(lt), "%3s %3s %d %d:%d:%d %d\n",
+	       day, mon, &dd, &hh, &mm, &ss, &yyyy);
+	g_snprintf(buf, len, "%s, %d %s %d %02d:%02d:%02d %s",
+		   day, dd, mon, yyyy, hh, mm, ss, tzoffset(&t));
+}
+
 void debug_print(const gchar *format, ...)
 {
 	va_list args;
@@ -891,6 +1363,45 @@ void log_print(const gchar *format, ...)
 	va_end(args);
 
 	if (debug_mode) fputs(buf, stdout);
-	log_window_append(buf);
+	log_window_append(buf, LOG_NORMAL);
 	statusbar_puts_all(buf);
 }
+
+void log_message(const gchar *format, ...)
+{
+	va_list args;
+	gchar buf[BUFFSIZE];
+
+	va_start(args, format);
+	g_vsnprintf(buf, sizeof(buf), format, args);
+	va_end(args);
+
+	if (debug_mode) g_message("%s", buf);
+	log_window_append(buf, LOG_MSG);
+}
+
+void log_warning(const gchar *format, ...)
+{
+	va_list args;
+	gchar buf[BUFFSIZE];
+
+	va_start(args, format);
+	g_vsnprintf(buf, sizeof(buf), format, args);
+	va_end(args);
+
+	g_warning("%s", buf);
+	log_window_append(buf, LOG_WARN);
+}
+
+void log_error(const gchar *format, ...)
+{
+	va_list args;
+	gchar buf[BUFFSIZE];
+
+	va_start(args, format);
+	g_vsnprintf(buf, sizeof(buf), format, args);
+	va_end(args);
+
+	g_warning("%s", buf);
+	log_window_append(buf, LOG_ERROR);
+}
